;*****************************************************************************
;				NICST TOOL
;
;  RC-05  test for TEB bands. Calculates the background subtracted dn and
;  standard deviations.
;
;  Input:
;  1) File with inpath and outpath.
;
;  Output:
;  1) dn(collect,band,HAM,detector,subsample,gain,3] -- dn for frame average then scan average,
;     scan average then frame average, and average over all scans and frames 
;     with a 3 sigma kick out at each step.
;  2) std(collect,band,HAM,detector,subsample,gain,3] -- std for frame average then scan average,
;     scan average then frame average, and average over all scans and frames 
;     with a 3 sigma kick out at each step.
;  3) dn_col(collect,band,HAM,detector,subsample,gain,scan,3] -- dn for frame average with a 3 sigma kick 
;     out at each step for (0=EV, 1=SV, 2=OBC). The EV and OBC indices are SV subtracted.
;  4) std_col(collect,band,HAM,detector,subsample,gain,scan,3] -- std for frame average with a 3 sigma kick 
;     out at each step for (0=EV, 1=SV, 2=OBC). The EV and OBC indices are SV subtracted.
;  5) dn_col_2(collect,band,HAM,detector,subsample,gain,frames,3] -- dn for scan average with a 3 sigma kick 
;     out at each step for (0=EV, 1=OBC). The EV and OBC indices are SV subtracted.
;  6) std_col_2(collect,band,HAM,detector,subsample,gain,frames,3] -- std for scan average with a 3 sigma kick 
;     out at each step for (0=EV, 1=OBC). The EV and OBC indices are SV subtracted.
;  7) dn_obc(collect,band,HAM,detector,subsample,gain,3] -- dn for frame average then scan average,
;     scan average then frame average, and average over all scans and frames 
;     with a 3 sigma kick out at each step.
;  8) std_obc(collect,band,HAM,detector,subsample,gain,3] -- std for frame average then scan average,
;     scan average then frame average, and average over all scans and frames 
;     with a 3 sigma kick out at each step.
;  9) DN_EV(collect,band,HAM,detector,subsample,gain,scan) -- EV DN frame averaged.
;  10) std_EV(collect,band,HAM,detector,subsample,gain,scan) -- standard deviation of EV DN.
;  11) DN_OBC(collect,band,HAM,detector,subsample,gain,scan) -- OBC DN frame averaged.
;  12) std_OBC(collect,band,HAM,detector,subsample,gain,scan) -- standard deviation of EV DN.
;  13) slope1=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan,2) -- 
;     slope of dn over frame.
;  14) slope2=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2) -- 
;     slope of dn over scans (frame averaged).
;  15) slope3=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2) -- 
;     slope of dn over frame (scan averaged).
;  16) slope_obc1=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan,2) -- 
;     slope of OBC dn over frame.
;  17) slope_obc2=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2) -- 
;     slope of dn over scans (frame averaged).
;  18) slope_obc3=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2) -- 
;     slope of dn over frame (scan averaged).
;  19) slope_sv2=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2) -- 
;     slope of dn over scans (frame averaged).
;  
;  v1.0 - 12/19/2008 - Jeff McIntire
;*****************************************************************************

PRO RC5_dn_3, infile

;*****************************************************************************
;  Read in input file, sav TEB file, and get list of dump files.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.
maxscan=intarr(1)               ; # of scans
sample1=intarr(1)               ; 1st M band sample
nsamples=intarr(1)              ; # of M band samples

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,maxscan
readf,lun,sample1
readf,lun,nsamples
Free_Lun,lun

max_scan=maxscan[0]
max_frame=nsamples[0]

; Number of files missing.
missing=0

; Number of types of files (EV, SV, etc)
n_type=3

restore,'general.TEB.2.sav'

; Get list of .dmp files from specified path. 
spawn, 'ls -C1 ' + inpath + '*EV*.dmp', list_files1
spawn, 'ls -C1 ' + inpath + '*SV*.dmp', list_files2
spawn, 'ls -C1 ' + inpath + '*OB*.dmp', list_files3
IF N_ELEMENTS(list_files1) EQ 0 THEN goto,skipout
nfile=N_ELEMENTS(list_files1)

ncollect=nfile/(nbands-missing)
ncollects=ncollect[0]

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)
outfile=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

; DN values per collect.
DN_EV=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan)
DN_OB=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan)

; std values per collect.
std_EV=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan)
std_OB=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan)

; dn values per collect (0=EV, 1=SV, 2=OBC).
dn_col=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan,3)
dn_col_2=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2*max_frame,3)

; Final dn (0=frame,scan 1=scan,frame 2frame+scan).
dn=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,3)
dn_obc=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,3)

; STD values per collect (0=EV, 1=SV, 2=OBC).
std_col=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan,3)
std_col_2=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2*max_frame,3)

; Final std (0=frame,scan 1=scan,frame 2frame+scan).
std=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,3)
std_obc=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,3)

; Slopes of EV and OBC over frame
slope1=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan,2)
slope_obc1=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan,2)

; Slopes of EV, SV, and OBC over scans (frame averaged)
slope2=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2)
slope_sv2=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2)
slope_obc2=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2)

; Slopes of EV and OBC over frames (scan averaged)
slope3=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2)
slope_obc3=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,2)

;*****************************************************************************
;  Read in the dump files for SV.
;*****************************************************************************

m=0
a5=0
FOR ifile=0, nfile-1 DO BEGIN
    nlines=File_Lines(list_files2[ifile])
    IF nlines EQ 0 THEN continue
    
    a2=strsplit(list_files2[ifile],'/',/Extract)
    a3=N_ELEMENTS(a2)
    a8=strmid(a2[a3-1],strpos(a2[a3-1],'/',/Reverse_Search)+10,2) ; Determine EV or SV.

    ; Keep only the SV.
    IF a8 NE 'SV' THEN continue

    ; Determine the relative collect #.
    m=m+1
    IF m EQ (nbands-missing[0])+1 THEN BEGIN
        a5=a5+1
        m=1
    ENDIF
    print,list_files2[ifile],' ',a5
    
    ; Read in dump file.
    read_dmpd3,list_files2[ifile],header,uaid,collect,scan,ham,apid,band,dnb_s,det,dsect,frame,offset,gain_type,dns
    nlines=nlines-header[0]

    ; Translate the band name to a number.
    FOR ib=0, nbands-1 DO BEGIN
        IF band[0] EQ band_name[ib] THEN b=ib
    ENDFOR

    ; Number of frames.
    max_frame=max(frame)-min(frame)+1
    
;*****************************************************************************
;  Write DN values to an array for SV.
;*****************************************************************************

    dn_frame=dblarr(nham,maxdet,maxsubsets,maxgain,max_scan,max_frame)
    iscan=0
    idet=0
    iframe=0
    subset=0
    FOR i=0L, nlines-1 DO BEGIN
        IF scan[i] EQ iscan+1 AND det[i] EQ idet+1 THEN BEGIN
            dn_frame[ham[i],idet,subset,gain_type[i],iscan,iframe]=dns[i]
            iframe=iframe+1
            IF iframe EQ max_frame THEN BEGIN
                idet=idet+1
                iframe=0
                IF idet EQ detectors[b] THEN BEGIN
                    idet=0
                    iscan=iscan+1
                ENDIF
            ENDIF
        ENDIF
        IF band_type[b] EQ 1 THEN BEGIN
            IF subset EQ 0 THEN BEGIN
                subset=1
                goto,skip_over_1
            ENDIF
            IF subset EQ 1 THEN subset=0
            skip_over_1:
        ENDIF
    ENDFOR

;*****************************************************************************
;  Average over frames and calculate the standard deviation.
;*****************************************************************************

    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors[b]-1 DO BEGIN
            FOR isubset=0, subsets[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    FOR iscan=0, max_scan-1 DO BEGIN
                        dn_col[a5,b,iham,idet,isubset,igain,iscan,1]= $
                          sig_mean(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1],3,1)
                        std_col[a5,b,iham,idet,isubset,igain,iscan,1]= $
                          sig_mean(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1],3,2)
                        
                        IF dn_col[a5,b,iham,idet,isubset,igain,iscan,1] EQ -999 THEN $
                          dn_col[a5,b,iham,idet,isubset,igain,iscan,1]=0
                        IF std_col[a5,b,iham,idet,isubset,igain,iscan,1] EQ -999 THEN $
                          std_col[a5,b,iham,idet,isubset,igain,iscan,1]=0
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR

;*****************************************************************************
;  Relabel the higher subsample as 0 and the lower as 1.
;*****************************************************************************

    temp1=dblarr(nham,maxdet,maxsubsets,2,max_scan)
    IF band_type[b] EQ 1 THEN BEGIN
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    FOR iscan=0, max_scan-1 DO BEGIN
                        IF dn_col[a5,b,iham,idet,0,igain,iscan,1] $
                          GT dn_col[a5,b,iham,idet,1,igain,iscan,1] THEN BEGIN
                            temp1[iham,idet,0,igain,iscan]=dn_col[a5,b,iham,idet,0,igain,iscan,1]
                            temp1[iham,idet,1,igain,iscan]=dn_col[a5,b,iham,idet,1,igain,iscan,1]
                        ENDIF ELSE BEGIN
                            temp1[iham,idet,0,igain,iscan]=dn_col[a5,b,iham,idet,1,igain,iscan,1]
                            temp1[iham,idet,1,igain,iscan]=dn_col[a5,b,iham,idet,0,igain,iscan,1]
                        ENDELSE
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
        dn_col[a5,b,*,*,0,*,*,1]=temp1[*,*,0,*,*]
        dn_col[a5,b,*,*,1,*,*,1]=temp1[*,*,1,*,*]
    ENDIF

;*****************************************************************************
;  Calculate drifting of dn over scans (where the dn has been frame averaged) 
;  using a straight line fit.
;*****************************************************************************

    g=findgen(max_scan)
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors[b]-1 DO BEGIN
            FOR isubset=0, subsets[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    result2a=poly_fit(g[0:max_scan-1],dn_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,1],1)
                    
                    slope_sv2[a5,b,iham,idet,isubset,igain,*]=result2a[*]
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Determine if auto gain or fixed high gain for M13, detector 1 using
;  the first four scans of collect 1.
;*****************************************************************************

IF dn_col[0,3,0,0,0,0,0,1] EQ 0 THEN j1=0 ELSE j1=1
IF dn_col[0,3,0,0,0,0,1,1] EQ 0 THEN j2=0 ELSE j2=1
IF dn_col[0,3,0,0,0,0,2,1] EQ 0 THEN j3=0 ELSE j3=1
IF dn_col[0,3,0,0,0,0,3,1] EQ 0 THEN j4=0 ELSE j4=1

; if all zero then fixed high gain, if one non-zero then suto gain,
; and if two non-zero then fixed low gain.
IF j1+j2+j3+j4 EQ 0 THEN type_gain=0
IF j1+j2+j3+j4 EQ 1 THEN type_gain=1
IF j1+j2+j3+j4 EQ 2 THEN type_gain=2

; Auto Gain: write low background to index with high background and vice versa.
IF type_gain EQ 1 THEN BEGIN
    FOR col=0, ncollects-1 DO BEGIN
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                FOR iscan=0, max_scan-1 DO BEGIN
                    IF iscan+4 LT max_scan-1 THEN BEGIN
                        IF dn_col[col,3,iham,idet,0,0,iscan,1] GT 0 AND dn_col[col,3,iham,idet,0,0,iscan+4,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,0,iscan+2,1]=dn_col[col,3,iham,idet,0,0,iscan,1]
                        IF dn_col[col,3,iham,idet,0,1,iscan,1] GT 0 AND dn_col[col,3,iham,idet,0,1,iscan+4,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,1,iscan+2,1]=dn_col[col,3,iham,idet,0,1,iscan,1]
                    ENDIF
                    IF iscan LE 1 THEN BEGIN
                        IF dn_col[col,3,iham,idet,0,0,iscan+2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,0,iscan,1]=dn_col[col,3,iham,idet,0,0,iscan+2,1]
                        IF dn_col[col,3,iham,idet,0,1,iscan+2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,1,iscan,1]=dn_col[col,3,iham,idet,0,1,iscan+2,1]
                    ENDIF
                    IF iscan GE max_scan-3 THEN BEGIN
                        IF dn_col[col,3,iham,idet,0,0,iscan-2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,0,iscan,1]=dn_col[col,3,iham,idet,0,0,iscan-2,1]
                        IF dn_col[col,3,iham,idet,0,1,iscan-2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,1,iscan,1]=dn_col[col,3,iham,idet,0,1,iscan-2,1]
                    ENDIF
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDIF

;*****************************************************************************
;  Read in the dump files for EV.
;*****************************************************************************

FOR j=0, 1 DO BEGIN
    q=0
    a5=0
    FOR ifile=0, nfile-1 DO BEGIN
        IF j EQ 0 THEN nlines=File_Lines(list_files1[ifile])
        IF j EQ 1 THEN nlines=File_Lines(list_files3[ifile])
        IF nlines EQ 0 THEN continue
    
        IF j EQ 0 THEN a2=strsplit(list_files1[ifile],'/',/Extract)
        IF j EQ 1 THEN a2=strsplit(list_files3[ifile],'/',/Extract)
        a3=N_ELEMENTS(a2)
        a8=strmid(a2[a3-1],strpos(a2[a3-1],'/',/Reverse_Search)+10,2) ; Determine EV or SV.
        
        ; Keep only the EV and OBC.
        IF a8 EQ 'SV' THEN continue
        
        ; Determine the relative collect #.
        q=q+1
        IF q EQ (nbands-missing[0])+1 THEN BEGIN
            a5=a5+1
            q=1
        ENDIF
        IF j EQ 0 THEN print,list_files1[ifile],' ',a5
        IF j EQ 1 THEN print,list_files3[ifile],' ',a5
        
        IF j EQ 0 THEN read_dmpd3,list_files1[ifile],header,uaid,collect,scan,ham,apid,band,dnb_s,det,dsect,frame,offset,gain_type,dns
        IF j EQ 1 THEN read_dmpd3,list_files3[ifile],header,uaid,collect,scan,ham,apid,band,dnb_s,det,dsect,frame,offset,gain_type,dns
        nlines=nlines-header[0]

        ; Translate the band name to a number.
        FOR ib=0, nbands-1 DO BEGIN
            IF band[0] EQ band_name[ib] THEN b=ib
        ENDFOR
        
        ; Number of frames.
        max_frame=max(frame)-min(frame)+1

;*****************************************************************************
;  Write DN values to an array for EV.
;*****************************************************************************

        dn_frame=dblarr(nham,maxdet,maxsubsets,maxgain,max_scan,max_frame)
        iscan=0
        idet=0
        iframe=0
        subset=0
        FOR i=0L, nlines-1 DO BEGIN
            IF scan[i] EQ iscan+1 AND det[i] EQ idet+1 THEN BEGIN
                dn_frame[ham[i],idet,subset,gain_type[i],iscan,iframe]=dns[i]
                iframe=iframe+1
                IF iframe EQ max_frame THEN BEGIN
                    idet=idet+1
                    iframe=0
                    IF idet EQ detectors[b] THEN BEGIN
                        idet=0
                        iscan=iscan+1
                    ENDIF
                ENDIF
            ENDIF
            IF band_type[b] EQ 1 THEN BEGIN
                IF subset EQ 0 THEN BEGIN
                    subset=1
                    goto,skip_over_2
                ENDIF
                IF subset EQ 1 THEN subset=0
                skip_over_2:
            ENDIF
        ENDFOR
        
;*****************************************************************************
;  Subtract the SV average over frames from each frame of EV (keep label of
;  first subsample as 0 and second subsample as 1).
;*****************************************************************************

        dn_col_1=dblarr(nham,maxdet,maxsubsets,maxgain,max_scan,max_frame,2) ; (0=EV, 1=OBC)
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                FOR isubset=0, subsets[b]-1 DO BEGIN
                    FOR igain=0, 1 DO BEGIN
                        IF a8 EQ 'EV' THEN BEGIN
                            FOR iscan=0, max_scan-1 DO BEGIN
                                IF sig_mean(dn_frame[iham,idet,0,igain,iscan,0:max_frame-1],3,1) GT $
                                  sig_mean(dn_frame[iham,idet,1,igain,iscan,0:max_frame-1],3,1) THEN flag=0
                                IF sig_mean(dn_frame[iham,idet,0,igain,iscan,0:max_frame-1],3,1) LT $
                                  sig_mean(dn_frame[iham,idet,1,igain,iscan,0:max_frame-1],3,1) THEN flag=1
                                IF sig_mean(dn_frame[iham,idet,0,igain,iscan,0:max_frame-1],3,1) EQ -999 AND $
                                  sig_mean(dn_frame[iham,idet,1,igain,iscan,0:max_frame-1],3,1) EQ -999 THEN continue
                                FOR iframe=0, max_frame-1 DO BEGIN
                                    IF band_type[b] EQ 1 THEN BEGIN
                                        IF dn_frame[iham,idet,isubset,igain,iscan,iframe] NE 0 AND flag EQ 0 THEN BEGIN
                                            
                                            dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]= $
                                              dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                              -dn_col[a5,b,iham,idet,isubset,igain,iscan,1]
                                            dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]= $
                                              dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                              -dn_col[a5,b,iham,idet,isubset,igain,iscan,1]
                                        ENDIF
                                        IF dn_frame[iham,idet,isubset,igain,iscan,iframe] NE 0 AND flag EQ 1 THEN BEGIN
                                            
                                            IF isubset EQ 0 THEN dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]= $
                                              dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                              -dn_col[a5,b,iham,idet,1,igain,iscan,1]
                                            IF isubset EQ 1 THEN dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]= $
                                              dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                              -dn_col[a5,b,iham,idet,0,igain,iscan,1]
                                        ENDIF
                                        IF dn_frame[iham,idet,isubset,igain,iscan,iframe] EQ 0 THEN $
                                          dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]=0
                                    ENDIF
                                    IF band_type[b] EQ 0 THEN BEGIN
                                        IF dn_frame[iham,idet,isubset,igain,iscan,iframe] NE 0 THEN $
                                          dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]= $
                                          dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                          -dn_col[a5,b,iham,idet,isubset,igain,iscan,1]
                                        IF dn_frame[iham,idet,isubset,igain,iscan,iframe] EQ 0 THEN $
                                          dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]=0
                                    ENDIF
                                ENDFOR
                            ENDFOR
                        ENDIF
                        IF a8 EQ 'OB' THEN BEGIN
                            FOR iscan=0, max_scan-1 DO BEGIN
                                IF sig_mean(dn_frame[iham,idet,0,igain,iscan,0:max_frame-1],3,1) GT $
                                  sig_mean(dn_frame[iham,idet,1,igain,iscan,0:max_frame-1],3,1) THEN flag=0
                                IF sig_mean(dn_frame[iham,idet,0,igain,iscan,0:max_frame-1],3,1) LT $
                                  sig_mean(dn_frame[iham,idet,1,igain,iscan,0:max_frame-1],3,1) THEN flag=1
                                IF sig_mean(dn_frame[iham,idet,0,igain,iscan,0:max_frame-1],3,1) EQ -999 AND $
                                  sig_mean(dn_frame[iham,idet,1,igain,iscan,0:max_frame-1],3,1) EQ -999 THEN continue
                                FOR iframe=0, max_frame-1 DO BEGIN
                                    IF band_type[b] EQ 1 THEN BEGIN
                                        IF dn_frame[iham,idet,isubset,igain,iscan,iframe] NE 0 AND flag EQ 0 THEN BEGIN
                                            
                                            dn_col_1[iham,idet,isubset,igain,iscan,iframe,1]= $
                                              dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                              -dn_col[a5,b,iham,idet,isubset,igain,iscan,1]
                                            dn_col_1[iham,idet,isubset,igain,iscan,iframe,1]= $
                                              dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                              -dn_col[a5,b,iham,idet,isubset,igain,iscan,1]
                                        ENDIF
                                        IF dn_frame[iham,idet,isubset,igain,iscan,iframe] NE 0 AND flag EQ 1 THEN BEGIN
                                            
                                            IF isubset EQ 0 THEN dn_col_1[iham,idet,isubset,igain,iscan,iframe,1]= $
                                              dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                              -dn_col[a5,b,iham,idet,1,igain,iscan,1]
                                            IF isubset EQ 1 THEN dn_col_1[iham,idet,isubset,igain,iscan,iframe,1]= $
                                              dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                              -dn_col[a5,b,iham,idet,0,igain,iscan,1]
                                        ENDIF
                                        IF dn_frame[iham,idet,isubset,igain,iscan,iframe] EQ 0 THEN $
                                          dn_col_1[iham,idet,isubset,igain,iscan,iframe,1]=0
                                    ENDIF
                                    IF band_type[b] EQ 0 THEN BEGIN
                                        IF dn_frame[iham,idet,isubset,igain,iscan,iframe] NE 0 THEN $
                                          dn_col_1[iham,idet,isubset,igain,iscan,iframe,1]= $
                                          dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                          -dn_col[a5,b,iham,idet,isubset,igain,iscan,1]
                                        IF dn_frame[iham,idet,isubset,igain,iscan,iframe] EQ 0 THEN $
                                          dn_col_1[iham,idet,isubset,igain,iscan,iframe,1]=0
                                    ENDIF
                                ENDFOR
                            ENDFOR
                        ENDIF
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                FOR isubset=0, subsets[b]-1 DO BEGIN
                    FOR igain=0, 1 DO BEGIN
                        FOR iscan=0, max_scan-1 DO BEGIN
                            IF a8 EQ 'EV' THEN DN_EV[a5,b,iham,idet,isubset,igain,iscan]=sig_mean(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1],3,1)
                            IF a8 EQ 'EV' THEN std_EV[a5,b,iham,idet,isubset,igain,iscan]=sig_mean(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1],3,2)
                            IF a8 EQ 'OB' THEN DN_OB[a5,b,iham,idet,isubset,igain,iscan]=sig_mean(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1],3,1)
                            IF a8 EQ 'OB' THEN std_OB[a5,b,iham,idet,isubset,igain,iscan]=sig_mean(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1],3,2)
                        ENDFOR
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
        
;*****************************************************************************
;  Average over frames with standard deviation.
;*****************************************************************************

        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                FOR isubset=0, subsets[b]-1 DO BEGIN
                    FOR igain=0, 1 DO BEGIN
                        FOR iscan=0, max_scan-1 DO BEGIN
                            IF a8 EQ 'EV' THEN BEGIN
                                dn_col[a5,b,iham,idet,isubset,igain,iscan,0]= $
                                  sig_mean(dn_col_1[iham,idet,isubset,igain,iscan,0:max_frame-1,0],3,1)
                                std_col[a5,b,iham,idet,isubset,igain,iscan,0]= $
                                  sig_mean(dn_col_1[iham,idet,isubset,igain,iscan,0:max_frame-1,0],3,2)
                                IF dn_col[a5,b,iham,idet,isubset,igain,iscan,0] EQ -999 THEN $
                                  dn_col[a5,b,iham,idet,isubset,igain,iscan,0]=0
                                IF std_col[a5,b,iham,idet,isubset,igain,iscan,0] EQ -999 THEN $
                                  std_col[a5,b,iham,idet,isubset,igain,iscan,0]=0
                            ENDIF
                            IF a8 EQ 'OB' THEN BEGIN
                                dn_col[a5,b,iham,idet,isubset,igain,iscan,2]= $
                                  sig_mean(dn_col_1[iham,idet,isubset,igain,iscan,0:max_frame-1,1],3,1)
                                std_col[a5,b,iham,idet,isubset,igain,iscan,2]= $
                                  sig_mean(dn_col_1[iham,idet,isubset,igain,iscan,0:max_frame-1,1],3,2)
                                IF dn_col[a5,b,iham,idet,isubset,igain,iscan,2] EQ -999 THEN $
                                  dn_col[a5,b,iham,idet,isubset,igain,iscan,2]=0
                                IF std_col[a5,b,iham,idet,isubset,igain,iscan,2] EQ -999 THEN $
                                  std_col[a5,b,iham,idet,isubset,igain,iscan,2]=0
                            ENDIF
                        ENDFOR
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR

;*****************************************************************************
;  Average the dn over scans and average the standard deviations.
;*****************************************************************************

        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                FOR isubset=0, subsets[b]-1 DO BEGIN
                    FOR igain=0, 1 DO BEGIN
                        IF a8 EQ 'EV' THEN BEGIN
                            dn[a5,b,iham,idet,isubset,igain,0]=sig_mean(dn_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,0],3,1)
                            
                            x=WHERE(std_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,0] NE 0,num)
                            IF num EQ 1 THEN std[a5,b,iham,idet,isubset,igain,0]=mean(std_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,0])*2 ELSE $
                              std[a5,b,iham,idet,isubset,igain,0]=sig_mean(std_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,0],3,1)
                            
                            IF dn[a5,b,iham,idet,isubset,igain,0] EQ -999 THEN dn[a5,b,iham,idet,isubset,igain,0]=0
                            IF std[a5,b,iham,idet,isubset,igain,0] EQ -999 THEN std[a5,b,iham,idet,isubset,igain,0]=0
                        ENDIF
                        IF a8 EQ 'OB' THEN BEGIN
                            dn_obc[a5,b,iham,idet,isubset,igain,0]=sig_mean(dn_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,2],3,1)
                            
                            x=WHERE(std_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,2] NE 0,num)
                            IF num EQ 1 THEN std_obc[a5,b,iham,idet,isubset,igain,0]=mean(std_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,2])*2 ELSE $
                              std_obc[a5,b,iham,idet,isubset,igain,0]=sig_mean(std_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,2],3,1)
                            
                            IF dn_obc[a5,b,iham,idet,isubset,igain,0] EQ -999 THEN dn_obc[a5,b,iham,idet,isubset,igain,0]=0
                            IF std_obc[a5,b,iham,idet,isubset,igain,0] EQ -999 THEN std_obc[a5,b,iham,idet,isubset,igain,0]=0
                        ENDIF
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    
;*****************************************************************************
;  Calculate the dn and standard deviation over scans, then average over frames.
;*****************************************************************************

        dn_scan_avg=dblarr(nham,maxdet,maxsubsets,maxgain,max_frame,2) ; (0=EV and 1=OBC)
        std_scan_avg=dblarr(nham,maxdet,maxsubsets,maxgain,max_frame,2)
        dn_avg_over_frames=dblarr(nham,maxdet,maxsubsets,maxgain,2)
        std_avg_over_frames=dblarr(nham,maxdet,maxsubsets,maxgain,2)
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                FOR isubset=0, subsets[b]-1 DO BEGIN
                    FOR igain=0, 1 DO BEGIN
                        IF a8 EQ 'EV' THEN BEGIN
                            FOR iframe=0, max_frame-1 DO BEGIN
                                dn_scan_avg[iham,idet,isubset,igain,iframe,0]= $
                                  sig_mean(dn_col_1[iham,idet,isubset,igain,0:max_scan-1,iframe,0],3,1)
                                std_scan_avg[iham,idet,isubset,igain,iframe,0]= $
                                  sig_mean(dn_col_1[iham,idet,isubset,igain,0:max_scan-1,iframe,0],3,2)
                                
                                IF dn_scan_avg[iham,idet,isubset,igain,iframe,0] EQ -999 THEN $
                                  dn_scan_avg[iham,idet,isubset,igain,iframe,0]=0
                                IF std_scan_avg[iham,idet,isubset,igain,iframe,0] EQ -999 THEN $
                                  std_scan_avg[iham,idet,isubset,igain,iframe,0]=0
                            ENDFOR
                            dn_avg_over_frames[iham,idet,isubset,igain,0]= $
                              sig_mean(dn_scan_avg[iham,idet,isubset,igain,0:max_frame-1,0],3,1)
                            std_avg_over_frames[iham,idet,isubset,igain,0]= $
                              sig_mean(std_scan_avg[iham,idet,isubset,igain,0:max_frame-1,0],3,1)
                        ENDIF
                        IF a8 EQ 'OB' THEN BEGIN
                            FOR iframe=0, max_frame-1 DO BEGIN
                                dn_scan_avg[iham,idet,isubset,igain,iframe,1]= $
                                  sig_mean(dn_col_1[iham,idet,isubset,igain,0:max_scan-1,iframe,1],3,1)
                                std_scan_avg[iham,idet,isubset,igain,iframe,1]= $
                                  sig_mean(dn_col_1[iham,idet,isubset,igain,0:max_scan-1,iframe,1],3,2)
                                
                                IF dn_scan_avg[iham,idet,isubset,igain,iframe,1] EQ -999 THEN $
                                  dn_scan_avg[iham,idet,isubset,igain,iframe,1]=0
                                IF std_scan_avg[iham,idet,isubset,igain,iframe,1] EQ -999 THEN $
                                  std_scan_avg[iham,idet,isubset,igain,iframe,1]=0
                            ENDFOR
                            dn_avg_over_frames[iham,idet,isubset,igain,1]= $
                              sig_mean(dn_scan_avg[iham,idet,isubset,igain,0:max_frame-1,1],3,1)
                            std_avg_over_frames[iham,idet,isubset,igain,1]= $
                              sig_mean(std_scan_avg[iham,idet,isubset,igain,0:max_frame-1,1],3,1)
                        ENDIF
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
        IF a8 EQ 'EV' THEN dn[a5,b,*,*,*,*,1]=dn_avg_over_frames[*,*,*,*,0]
        IF a8 EQ 'EV' THEN std[a5,b,*,*,*,*,1]=std_avg_over_frames[*,*,*,*,0]
        IF a8 EQ 'EV' THEN dn_col_2[a5,b,*,*,*,*,0:max_frame-1,0]=dn_scan_avg[*,*,*,*,0:max_frame-1,0]
        IF a8 EQ 'EV' THEN std_col_2[a5,b,*,*,*,*,0:max_frame-1,0]=std_scan_avg[*,*,*,*,0:max_frame-1,0]
        IF a8 EQ 'OB' THEN dn_obc[a5,b,*,*,*,*,1]=dn_avg_over_frames[*,*,*,*,1]
        IF a8 EQ 'OB' THEN std_obc[a5,b,*,*,*,*,1]=std_avg_over_frames[*,*,*,*,1]
        IF a8 EQ 'OB' THEN dn_col_2[a5,b,*,*,*,*,0:max_frame-1,2]=dn_scan_avg[*,*,*,*,0:max_frame-1,1]
        IF a8 EQ 'OB' THEN std_col_2[a5,b,*,*,*,*,0:max_frame-1,2]=std_scan_avg[*,*,*,*,0:max_frame-1,1]

;*****************************************************************************
;  Calculate the dn and standard deviation over scans and frames.
;*****************************************************************************

        m=long(max_frame)
        n=long(max_scan)*long(max_frame)
        dn_total=dblarr(nham,maxdet,maxsubsets,maxgain,2) ; (0=EV and 1=OBC)
        std_total=dblarr(nham,maxdet,maxsubsets,maxgain,2)
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                FOR isubset=0, subsets[b]-1 DO BEGIN
                    FOR igain=0, 1 DO BEGIN
                        IF a8 EQ 'EV' THEN BEGIN
                            k=0
                            p=0L
                            temp=dblarr(n/2)
                            temp_obc=dblarr(n/2)
                            FOR i=0L, max_scan-1 DO BEGIN
                                IF dn_col_1[iham,idet,isubset,igain,i,0,0] NE 0 THEN BEGIN
                                    temp[k*m:(k+1)*m-1]=dn_col_1[iham,idet,isubset,igain,i,0:max_frame-1,0]
                                    k=k+1
                                ENDIF
                            ENDFOR
                            dn_total[iham,idet,isubset,igain,0]=sig_mean(temp[0:n/2-1],3,1)
                            std_total[iham,idet,isubset,igain,0]=sig_mean(temp[0:n/2-1],3,2)
                            
                            IF dn_total[iham,idet,isubset,igain,0] EQ -999 THEN dn_total[iham,idet,isubset,igain,0]=0
                            IF std_total[iham,idet,isubset,igain,0] EQ -999 THEN std_total[iham,idet,isubset,igain,0]=0
                        ENDIF
                        IF a8 EQ 'OB' THEN BEGIN
                            k=0
                            p=0L
                            temp=dblarr(n/2)
                            temp_obc=dblarr(n/2)
                            FOR i=0L, max_scan-1 DO BEGIN
                                IF dn_col_1[iham,idet,isubset,igain,i,0,1] NE 0 THEN BEGIN
                                    temp[k*m:(k+1)*m-1]=dn_col_1[iham,idet,isubset,igain,i,0:max_frame-1,1]
                                    k=k+1
                                ENDIF
                            ENDFOR
                            dn_total[iham,idet,isubset,igain,1]=sig_mean(temp[0:n/2-1],3,1)
                            std_total[iham,idet,isubset,igain,1]=sig_mean(temp[0:n/2-1],3,2)
                            
                            IF dn_total[iham,idet,isubset,igain,1] EQ -999 THEN dn_total[iham,idet,isubset,igain,1]=0
                            IF std_total[iham,idet,isubset,igain,1] EQ -999 THEN std_total[iham,idet,isubset,igain,1]=0
                        ENDIF
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
        IF a8 EQ 'EV' THEN dn[a5,b,*,*,*,*,2]=dn_total[*,*,*,*,0]
        IF a8 EQ 'EV' THEN std[a5,b,*,*,*,*,2]=std_total[*,*,*,*,0]
        IF a8 EQ 'OB' THEN dn_obc[a5,b,*,*,*,*,2]=dn_total[*,*,*,*,1]
        IF a8 EQ 'OB' THEN std_obc[a5,b,*,*,*,*,2]=std_total[*,*,*,*,1]

;*****************************************************************************
;  Calculate drifting of dn over frames using a straight line fit.
;*****************************************************************************

        f=findgen(max_frame)
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                FOR isubset=0, subsets[b]-1 DO BEGIN
                    FOR igain=0, 1 DO BEGIN
                        FOR iscan=0, max_scan-1 DO BEGIN
                            IF a8 EQ 'EV' THEN BEGIN
                                IF mean(dn_frame[iham,idet,isubset,igain,iscan,*]) EQ 0 THEN continue
                                
                                yy=WHERE(dn_frame[iham,idet,isubset,igain,iscan,*] NE 0)
                                IF n_elements(yy) LE 1 THEN continue
                                result1=poly_fit(f[0:n_elements(yy)-1],dn_frame[iham,idet,isubset,igain,iscan,yy],1)
                                
                                slope1[a5,b,iham,idet,isubset,igain,iscan,*]=result1[*]
                            ENDIF
                            IF a8 EQ 'OB' THEN BEGIN
                                IF mean(dn_frame[iham,idet,isubset,igain,iscan,*]) EQ 0 THEN continue
                                
                                yy=WHERE(dn_frame[iham,idet,isubset,igain,iscan,*] NE 0)
                                IF n_elements(yy) LE 1 THEN continue
                                result1a=poly_fit(f[0:n_elements(yy)-1],dn_frame[iham,idet,isubset,igain,iscan,yy],1)
                                
                                slope_obc1[a5,b,iham,idet,isubset,igain,iscan,*]=result1a[*]
                            ENDIF
                        ENDFOR
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR

;*****************************************************************************
;  Calculate drifting of dn over scans (where the dn has been frame averaged) 
;  using a straight line fit (scan method).
;*****************************************************************************

        g=findgen(max_scan)
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                FOR isubset=0, subsets[b]-1 DO BEGIN
                    FOR igain=0, 1 DO BEGIN
                        IF a8 EQ 'EV' THEN BEGIN
                            IF mean(dn_col[a5,b,iham,idet,isubset,igain,*,0]) EQ 0 THEN continue
                            
                            yy=WHERE(dn_col[a5,b,iham,idet,isubset,igain,*,0] NE 0)
                            IF n_elements(yy) LE 1 THEN continue
                            
                            result2=poly_fit(g[0:n_elements(yy)-1],dn_col[a5,b,iham,idet,isubset,igain,yy,0],1)
                            
                            slope2[a5,b,iham,idet,isubset,igain,*]=result2[*]
                        ENDIF
                        IF a8 EQ 'OB' THEN BEGIN 
                            IF mean(dn_col[a5,b,iham,idet,isubset,igain,*,2]) EQ 0 THEN continue
                            
                            yy=WHERE(dn_col[a5,b,iham,idet,isubset,igain,*,2] NE 0)
                            IF n_elements(yy) LE 1 THEN continue
                            
                            result2b=poly_fit(g[0:n_elements(yy)-1],dn_col[a5,b,iham,idet,isubset,igain,yy,2],1)
                            
                            slope_obc2[a5,b,iham,idet,isubset,igain,*]=result2b[*]
                        ENDIF
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR

;*****************************************************************************
;  Calculate drifting of dn over frames (where the dn has been scan averaged)
;  using a straight line fit (frame method).
;*****************************************************************************

        h=findgen(max_frame)
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                FOR isubset=0, subsets[b]-1 DO BEGIN
                    FOR igain=0, 1 DO BEGIN
                        IF a8 EQ 'EV' THEN BEGIN
                            IF mean(dn_col_2[a5,b,iham,idet,isubset,igain,*,0]) EQ 0 THEN continue
                            
                            yy=WHERE(dn_col_2[a5,b,iham,idet,isubset,igain,*,0] NE 0)
                            IF n_elements(yy) LE 1 THEN continue
                            result3=poly_fit(h[0:n_elements(yy)-1],dn_col_2[a5,b,iham,idet,isubset,igain,yy,0],1)
                            
                            slope3[a5,b,iham,idet,isubset,igain,*]=result3[*]
                        ENDIF
                        IF a8 EQ 'OB' THEN BEGIN
                            IF mean(dn_col_2[a5,b,iham,idet,isubset,igain,*,2]) EQ 0 THEN continue
                            
                            yy=WHERE(dn_col_2[a5,b,iham,idet,isubset,igain,*,2] NE 0)
                            IF n_elements(yy) LE 1 THEN continue
                            result3b=poly_fit(h[0:n_elements(yy)-1],dn_col_2[a5,b,iham,idet,isubset,igain,yy,2],1)
                            
                            slope_obc3[a5,b,iham,idet,isubset,igain,*]=result3b[*]
                        ENDIF
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Write to a save file.
;*****************************************************************************

save,/compress,dn,std,dn_obc,std_obc,dn_col,std_col,dn_col_2,std_col_2,filename=outfile+'.dn.sav'
save,/compress,DN_EV,DN_OBC,std_EV,std_OBC,slope1,slope2,slope3,slope_obc1,slope_obc2,slope_obc3,slope_sv2,filename=outfile+'.DN.slope.sav'

;*****************************************************************************
;  Set DN_EV plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outfile+'.DN_EV.ps'
device,/landscape,filename=outfname,/color

D=findgen(4000)
HAM=['A','B']
GAIN=['Low gain','High gain']

;*****************************************************************************
;  Plot DN for the EV over scan (scan method).
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR col=0, ncollects-1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        title='DN_EV vs Scan '+band_name[b]+' Collect '+strtrim(string(col+1),2)
        FOR iham=0, nham-1 DO BEGIN
            FOR isubset=0, subsets[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    IF mean(DN_EV[col,b,iham,0,isubset,igain,0:max_scan-1]) EQ -999 THEN continue
                    IF mean(DN_EV[col,b,iham,0,isubset,igain,0:max_scan-1]) EQ 0 THEN continue

                    title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)

                    zz=WHERE(DN_EV[col,b,iham,0,isubset,igain,0:max_scan-1] NE -999)

                    IF mean(DN_EV[col,b,iham,0,isubset,igain,zz]) LE -500 THEN continue

                    ymax=max(DN_EV[col,b,iham,0:detectors[b]-1,isubset,igain,zz],min=ymin)

                    plot,D[0:max_scan/2-1],DN_EV[col,b,iham,0,isubset,igain,zz],xtit='Scan',ytit='DN_EV',yrange=[ymin,ymax],title=title2, $
                      charsize=1.2,charthick=5,thick=5,/nodata
    
                    FOR idet=0, detectors[b]-1 DO BEGIN
                        oplot,D[0:max_scan/2-1],DN_EV[col,b,iham,idet,isubset,igain,zz],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                        oplot,D[0:max_scan/2-1],DN_EV[col,b,iham,idet,isubset,igain,zz],symsize=1,color=lcol[idet],thick=5
                    ENDFOR
                    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                    IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
                    IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

device,/close

;*****************************************************************************
;  Set DN_SV plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outfile+'.DN_SV.ps'
device,/landscape,filename=outfname,/color

;*****************************************************************************
;  Plot dn for the SV over scan (scan method).
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR col=0, ncollects-1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        title='DN_SV vs Scan '+band_name[b]+' Collect '+strtrim(string(col+1),2)
        FOR iham=0, nham-1 DO BEGIN
            FOR isubset=0, subsets[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    IF mean(dn_col[col,b,iham,0,isubset,igain,0:max_scan-1,1]) EQ 0 THEN continue

                    title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)

                    zz=WHERE(dn_col[col,b,iham,0,isubset,igain,0:max_scan-1,1] NE 0)

                    IF mean(dn_col[col,b,iham,0,isubset,igain,zz,1]) LE 0 THEN continue

                    ymax=max(dn_col[col,b,iham,0:detectors[b]-1,isubset,igain,zz,1],min=ymin)

                    plot,D[0:max_scan/2-1],dn_col[col,b,iham,0,isubset,igain,zz,1],xtit='Scan',ytit='DN_SV',yrange=[ymin,ymax],title=title2, $
                      charsize=1.2,charthick=5,thick=5,/nodata
    
                    FOR idet=0, detectors[b]-1 DO BEGIN
                        oplot,D[0:max_scan/2-1],dn_col[col,b,iham,idet,isubset,igain,zz,1],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                        oplot,D[0:max_scan/2-1],dn_col[col,b,iham,idet,isubset,igain,zz,1],symsize=1,color=lcol[idet],thick=5
                    ENDFOR
                    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                    IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
                    IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

device,/close

;*****************************************************************************
;  Set DN_OBC plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outfile+'.DN_OBC.ps'
device,/landscape,filename=outfname,/color

;*****************************************************************************
;  Plot dn for the OBC over scan (scan method).
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR col=0, ncollects-1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        title='DN_OBC vs Scan '+band_name[b]+' Collect '+strtrim(string(col+1),2)
        FOR iham=0, nham-1 DO BEGIN
            FOR isubset=0, subsets[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    IF mean(DN_OB[col,b,iham,0,isubset,igain,0:max_scan-1]) EQ -999 THEN continue
                    IF mean(DN_OB[col,b,iham,0,isubset,igain,0:max_scan-1]) EQ 0 THEN continue

                    title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)

                    zz=WHERE(DN_OB[col,b,iham,0,isubset,igain,0:max_scan-1] NE -999)

                    IF mean(DN_OB[col,b,iham,0,isubset,igain,zz]) LE -500 THEN continue

                    ymax=max(DN_OB[col,b,iham,0:detectors[b]-1,isubset,igain,zz],min=ymin)

                    plot,D[0:max_scan/2-1],DN_OB[col,b,iham,0,isubset,igain,zz],xtit='Scan',ytit='DN_OBC',yrange=[ymin,ymax],title=title2, $
                      charsize=1.2,charthick=5,thick=5,/nodata
    
                    FOR idet=0, detectors[b]-1 DO BEGIN
                        oplot,D[0:max_scan/2-1],DN_OB[col,b,iham,idet,isubset,igain,zz],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                        oplot,D[0:max_scan/2-1],DN_OB[col,b,iham,idet,isubset,igain,zz],symsize=1,color=lcol[idet],thick=5
                    ENDFOR
                    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                    IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
                    IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

device,/close

;*****************************************************************************
;  Set slope of EV dn over frames plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outfile+'.dn_slope_EV.ps'
device,/landscape,filename=outfname,/color

;*****************************************************************************
;  Plot slope of EV dn over frames (scan method).
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR col=0, ncollects-1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        title='Slope of DN_EV over frames vs Scan '+band_name[b]+' Collect '+strtrim(string(col+1),2)
        FOR iham=0, nham-1 DO BEGIN
            FOR isubset=0, subsets[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    IF mean(slope1[col,b,iham,0,isubset,igain,0:max_scan-1,1]) EQ 0 THEN continue

                    title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)

                    zz=WHERE(slope1[col,b,iham,0,isubset,igain,0:max_scan-1,1] NE 0)

                    ymax=max(slope1[col,b,iham,0:detectors[b]-1,isubset,igain,zz,1],min=ymin)

                    plot,D[0:max_scan/2-1],slope1[col,b,iham,0,isubset,igain,zz,1],xtit='Scan',ytit='Slope of dn_EV',yrange=[ymin,ymax],title=title2, $
                      charsize=1.2,charthick=5,thick=5,/nodata
    
                    FOR idet=0, detectors[b]-1 DO BEGIN
                        oplot,D[0:max_scan/2-1],slope1[col,b,iham,idet,isubset,igain,zz,1],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                        oplot,D[0:max_scan/2-1],slope1[col,b,iham,idet,isubset,igain,zz,1],symsize=1,color=lcol[idet],thick=5
                    ENDFOR
                    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                    IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
                    IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

device,/close

;*****************************************************************************
;  Set slope of OBC dn over frames plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outfile+'.dn_slope_OBC.ps'
device,/landscape,filename=outfname,/color

;*****************************************************************************
;  Plot slope of OBC dn over frames (scan method).
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR col=0, ncollects-1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        title='Slope of DN_OBC over frames vs Scan '+band_name[b]+' Collect '+strtrim(string(col+1),2)
        FOR iham=0, nham-1 DO BEGIN
            FOR isubset=0, subsets[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    IF mean(slope_obc1[col,b,iham,0,isubset,igain,0:max_scan-1,1]) EQ 0 THEN continue

                    title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)

                    zz=WHERE(slope_obc1[col,b,iham,0,isubset,igain,0:max_scan-1,1] NE 0)

                    ymax=max(slope_obc1[col,b,iham,0:detectors[b]-1,isubset,igain,zz,1],min=ymin)

                    plot,D[0:max_scan/2-1],slope_obc1[col,b,iham,0,isubset,igain,zz,1],xtit='Scan',ytit='Slope of dn_OBC',yrange=[ymin,ymax],title=title2, $
                      charsize=1.2,charthick=5,thick=5,/nodata
    
                    FOR idet=0, detectors[b]-1 DO BEGIN
                        oplot,D[0:max_scan/2-1],slope_obc1[col,b,iham,idet,isubset,igain,zz,1],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                        oplot,D[0:max_scan/2-1],slope_obc1[col,b,iham,idet,isubset,igain,zz,1],symsize=1,color=lcol[idet],thick=5
                    ENDFOR
                    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                    IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
                    IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

device,/close

;*****************************************************************************
;  Set slope of EV dn over scans, frame averaged plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outfile+'.dn_slope_EV_scan.ps'
device,/landscape,filename=outfname,/color

;*****************************************************************************
;  Plot slope of EV dn over scans, frame averaged (scan method).
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR b=0, nbands-1 DO BEGIN
    title='Slope of DN_EV over scans (frame averaged) vs Scan '+band_name[b]
    FOR iham=0, nham-1 DO BEGIN
        FOR isubset=0, subsets[b]-1 DO BEGIN
            FOR igain=0, 1 DO BEGIN
                IF mean(slope2[0:ncollects-1,b,iham,0,isubset,igain,1]) EQ 0 THEN continue
                
                title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)
                
                ymax=max(slope2[0:ncollects-1,b,iham,0:detectors[b]-1,isubset,igain,1],min=ymin)
                
                plot,D[0:ncollects-1],slope2[0:ncollects-1,b,iham,0,isubset,igain,1],xtit='Collect',ytit='Slope of DN_EV',yrange=[ymin,ymax],title=title2, $
                  charsize=1.2,charthick=5,thick=5,/nodata
                
                FOR idet=0, detectors[b]-1 DO BEGIN
                    oplot,D[0:ncollects-1],slope2[0:ncollects-1,b,iham,idet,isubset,igain,1],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                    oplot,D[0:ncollects-1],slope2[0:ncollects-1,b,iham,idet,isubset,igain,1],symsize=1,color=lcol[idet],thick=5
                ENDFOR
                XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
                IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

device,/close

;*****************************************************************************
;  Set slope of OBC dn over scans, frame averaged plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outfile+'.dn_slope_OBC_scan.ps'
device,/landscape,filename=outfname,/color

;*****************************************************************************
;  Plot slope of OBC dn over scans, frame averaged (scan method).
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR b=0, nbands-1 DO BEGIN
    title='Slope of DN_OBC over scans (frame averaged) vs Scan '+band_name[b]
    FOR iham=0, nham-1 DO BEGIN
        FOR isubset=0, subsets[b]-1 DO BEGIN
            FOR igain=0, 1 DO BEGIN
                IF mean(slope_obc2[0:ncollects-1,b,iham,0,isubset,igain,1]) EQ 0 THEN continue
                
                title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)
                
                ymax=max(slope_obc2[0:ncollects-1,b,iham,0:detectors[b]-1,isubset,igain,1],min=ymin)
                
                plot,D[0:ncollects-1],slope_obc2[0:ncollects-1,b,iham,0,isubset,igain,1],xtit='Collect',ytit='Slope of DN_OBC',yrange=[ymin,ymax],title=title2, $
                  charsize=1.2,charthick=5,thick=5,/nodata
                
                FOR idet=0, detectors[b]-1 DO BEGIN
                    oplot,D[0:ncollects-1],slope_obc2[0:ncollects-1,b,iham,idet,isubset,igain,1],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                    oplot,D[0:ncollects-1],slope_obc2[0:ncollects-1,b,iham,idet,isubset,igain,1],symsize=1,color=lcol[idet],thick=5
                ENDFOR
                XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
                IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

device,/close

;*****************************************************************************
;  Set EV standard deviation plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outfile+'.sigma_EV.ps'
device,/landscape,filename=outfname,/color

D=findgen(4000)
HAM=['A','B']
GAIN=['Low gain','High gain']

;*****************************************************************************
;  Plot standard deviation of EV over scan (scan method).
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR col=0, ncollects-1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        title='sigma_EV vs Scan '+band_name[b]+' Collect '+strtrim(string(col+1),2)
        FOR iham=0, nham-1 DO BEGIN
            FOR isubset=0, subsets[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    IF mean(std_EV[col,b,iham,0,isubset,igain,0:max_scan-1]) EQ -999 THEN continue
                    IF mean(std_EV[col,b,iham,0,isubset,igain,0:max_scan-1]) EQ 0 THEN continue

                    title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)

                    zz=WHERE(std_EV[col,b,iham,0,isubset,igain,0:max_scan-1] NE -999)

                    IF mean(std_EV[col,b,iham,0,isubset,igain,zz]) LE -500 THEN continue

                    ymax=max(std_EV[col,b,iham,0:detectors[b]-1,isubset,igain,zz],min=ymin)

                    plot,D[0:max_scan/2-1],std_EV[col,b,iham,0,isubset,igain,zz],xtit='Scan',ytit='sigma_EV',yrange=[ymin,ymax],title=title2, $
                      charsize=1.2,charthick=5,thick=5,/nodata
    
                    FOR idet=0, detectors[b]-1 DO BEGIN
                        oplot,D[0:max_scan/2-1],std_EV[col,b,iham,idet,isubset,igain,zz],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                        oplot,D[0:max_scan/2-1],std_EV[col,b,iham,idet,isubset,igain,zz],symsize=1,color=lcol[idet],thick=5
                    ENDFOR
                    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                    IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
                    IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

device,/close

;*****************************************************************************
;  Set SV standard deviation plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outfile+'.sigma_SV.ps'
device,/landscape,filename=outfname,/color

D=findgen(4000)
HAM=['A','B']
GAIN=['Low gain','High gain']

;*****************************************************************************
;  Plot standard deviation of SV over scan (scan method).
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR col=0, ncollects-1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        title='sigma_SV vs Scan '+band_name[b]+' Collect '+strtrim(string(col+1),2)
        FOR iham=0, nham-1 DO BEGIN
            FOR isubset=0, subsets[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    IF mean(std_col[col,b,iham,0,isubset,igain,0:max_scan-1]) EQ 0 THEN continue

                    title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)

                    zz=WHERE(std_col[col,b,iham,0,isubset,igain,0:max_scan-1] NE 0)

                    IF mean(std_col[col,b,iham,0,isubset,igain,zz]) LE 0 THEN continue

                    ymax=max(std_col[col,b,iham,0:detectors[b]-1,isubset,igain,zz],min=ymin)

                    plot,D[0:max_scan/2-1],std_col[col,b,iham,0,isubset,igain,zz],xtit='Scan',ytit='sigma_SV',yrange=[ymin,ymax],title=title2, $
                      charsize=1.2,charthick=5,thick=5,/nodata
    
                    FOR idet=0, detectors[b]-1 DO BEGIN
                        oplot,D[0:max_scan/2-1],std_col[col,b,iham,idet,isubset,igain,zz],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                        oplot,D[0:max_scan/2-1],std_col[col,b,iham,idet,isubset,igain,zz],symsize=1,color=lcol[idet],thick=5
                    ENDFOR
                    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                    IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
                    IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

device,/close

;*****************************************************************************
;  Set OBC standard deviation plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outfile+'.sigma_OBC.ps'
device,/landscape,filename=outfname,/color

D=findgen(4000)
HAM=['A','B']
GAIN=['Low gain','High gain']

;*****************************************************************************
;  Plot standard deviation of OBC over scan (scan method).
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR col=0, ncollects-1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        title='sigma_OBC vs Scan '+band_name[b]+' Collect '+strtrim(string(col+1),2)
        FOR iham=0, nham-1 DO BEGIN
            FOR isubset=0, subsets[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    IF mean(std_OB[col,b,iham,0,isubset,igain,0:max_scan-1]) EQ -999 THEN continue
                    IF mean(std_OB[col,b,iham,0,isubset,igain,0:max_scan-1]) EQ 0 THEN continue

                    title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)

                    zz=WHERE(std_OB[col,b,iham,0,isubset,igain,0:max_scan-1] NE -999)

                    IF mean(std_OB[col,b,iham,0,isubset,igain,zz]) LE -500 THEN continue

                    ymax=max(std_OB[col,b,iham,0:detectors[b]-1,isubset,igain,zz],min=ymin)

                    plot,D[0:max_scan/2-1],std_OB[col,b,iham,0,isubset,igain,zz],xtit='Scan',ytit='sigma_OBC',yrange=[ymin,ymax],title=title2, $
                      charsize=1.2,charthick=5,thick=5,/nodata
    
                    FOR idet=0, detectors[b]-1 DO BEGIN
                        oplot,D[0:max_scan/2-1],std_OB[col,b,iham,idet,isubset,igain,zz],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                        oplot,D[0:max_scan/2-1],std_OB[col,b,iham,idet,isubset,igain,zz],symsize=1,color=lcol[idet],thick=5
                    ENDFOR
                    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                    IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
                    IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

device,/close

skipout:

END

