;*****************************************************************************
;				NICST TOOL
;
;  Uses polynominal fits, linear (ifit=0),  quadratic (ifit=1) 
;  and cubic (ifit=2), relating dn to radiance. Uses the sample method for dn.
;  Also fit both with RSR (rsr=0) and without (rsr=1).
;
;  Input:
;  1) Input file.
;
;  Output:
;  1) ncollects
;  3) rg_1[band,HAM,detector,subsample,a0 a1 a2 a3,ifit,rsr] -- radiometric gain.
;  4) dL_fit[collect,band,HAM,detector,subsample,ifit,rsr] -- fractional fitting residual.
;  5) rrcu[band,HAM,detector,subsample,ifit,rsr,sum or sum in quadriture] 
;      -- radiometric response characterization uncertainty.
;  6) rrnl[band,HAM,detector,subsample,ifit,rsr] -- radiometric response non-linearity.
;  
;  v1.0 - 03/27/2008 - Jeff McIntire
;  v2.0 - 05/27/2008 - Jeff McIntire
;*****************************************************************************

PRO RC5_fit, infile

;*****************************************************************************
;  Read in input file, sav TEB file, and make some definitions.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)
param_in=strarr(1)
rsrpath=strarr(1)
fnumber=fltarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
Free_Lun,lun

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'

restore,'general.TEB.sav'

outfile=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.sav'

;*****************************************************************************
;  Definitions.
;*****************************************************************************

x=dblarr(ncollects)
z=dblarr(ncollects)

rg_1=dblarr(nbands_TEB,nham,maxdet,maxsubsets,4,6,2)
dL_fit=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,6,2)
mean_dL_fit=dblarr(nbands_TEB,nham,maxdet,maxsubsets,6,2)
abs_max_dL_fit=dblarr(nbands_TEB,nham,maxdet,maxsubsets,6,2)
stddev_dL_fit=dblarr(nbands_TEB,nham,maxdet,maxsubsets,6,2)
rrcu=dblarr(nbands_TEB,nham,maxdet,maxsubsets,6,2,2)
rrnl=dblarr(nbands_TEB,nham,maxdet,maxsubsets,6,2)

mindet=[0,0,0,0,1,0,0,0]

;*****************************************************************************
;  Fitting using POLY_FIT (Linear,Quadratic, and Cubic).
;  ifit=0 for linear, 1 for quadratic, and 2 for cubic POLY_FIT L vs dn (calibration equation).
;  ifit=3 for linear, 4 for quadratic, and 5 for cubic POLY_FIT dn vs L (response equation).
;*****************************************************************************

FOR ifit=0, 5 DO BEGIN
    ;  0 for rsr correction and 1 for no rsr correction.
    FOR rsr=0, 1 DO BEGIN
        FOR b=0, nbands_TEB-1 DO BEGIN
            FOR is=0, subsets_TEB[b]-1 DO BEGIN
                FOR ih=0, nham-1 DO BEGIN
                    FOR id=0, detectors_TEB[b]-1 DO BEGIN
                        ; Use only points within the dynamic range.
                        dyr=WHERE(rad_BCS[*,b,ih,id,is,rsr]*0.4444 GT L_min_TEB[b] AND rad_BCS[*,b,ih,id,is,rsr]*0.4444 LT L_max_TEB[b])

                        x=dn[dyr,b,ih,id,is,1,0]
                        z=rad_BCS[dyr,b,ih,id,is,rsr]*0.4444
                        
                        ; Calbration equation.
                        IF ifit EQ 0 THEN result1=POLY_FIT(x,z,1)
                        IF ifit EQ 1 THEN result2=POLY_FIT(x,z,2)
                        IF ifit EQ 2 THEN result3=POLY_FIT(x,z,3)

                        ; Response equation.
                        IF ifit EQ 3 THEN result1=POLY_FIT(z,x,1)
                        IF ifit EQ 4 THEN result2=POLY_FIT(z,x,2)
                        IF ifit EQ 5 THEN result3=POLY_FIT(z,x,3)
                        
                        IF ifit EQ 0 OR ifit EQ 3 THEN rg_1[b,ih,id,is,0:1,ifit,rsr]=result1[0:1]
                        IF ifit EQ 1 OR ifit EQ 4 THEN rg_1[b,ih,id,is,0:2,ifit,rsr]=result2[0:2]
                        IF ifit EQ 2 OR ifit EQ 5 THEN rg_1[b,ih,id,is,*,ifit,rsr]=result3[*]

;*****************************************************************************
;  Get the fractional fitting residual for the calibration equation
;  and the fitting residual for the response equation.
;*****************************************************************************

                        IF ifit EQ 0 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (rad_BCS[dyr,b,ih,id,is,rsr]*0.4444-result1[0]-result1[1]*dn[dyr,b,ih,id,is,1,0])/(rad_BCS[dyr,b,ih,id,is,rsr]*0.4444)
                        IF ifit EQ 1 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (rad_BCS[dyr,b,ih,id,is,rsr]*0.4444-result2[0]-result2[1]*dn[dyr,b,ih,id,is,1,0] $
                           -result2[2]*dn[dyr,b,ih,id,is,1,0]^2)/(rad_BCS[dyr,b,ih,id,is,rsr]*0.4444)
                        IF ifit EQ 2 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (rad_BCS[dyr,b,ih,id,is,rsr]*0.4444-result3[0]-result3[1]*dn[dyr,b,ih,id,is,1,0]-result3[2]*dn[dyr,b,ih,id,is,1,0]^2 $
                           -result3[3]*dn[dyr,b,ih,id,is,1,0]^3)/(rad_BCS[dyr,b,ih,id,is,rsr]*0.4444)
                        IF ifit EQ 3 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (dn[dyr,b,ih,id,is,1,0]-result1[0]-result1[1]*rad_BCS[dyr,b,ih,id,is,rsr]*0.4444)
                        IF ifit EQ 4 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (dn[dyr,b,ih,id,is,1,0]-result2[0]-result2[1]*rad_BCS[dyr,b,ih,id,is,rsr]*0.4444-result2[2]*(rad_BCS[dyr,b,ih,id,is,rsr]*0.4444)^2)
                        IF ifit EQ 5 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (dn[dyr,b,ih,id,is,1,0]-result3[0]-result3[1]*rad_BCS[dyr,b,ih,id,is,rsr]*0.4444-result3[2]*(rad_BCS[dyr,b,ih,id,is,rsr]*0.4444)^2 $
                           -result3[3]*(rad_BCS[dyr,b,ih,id,is,rsr]*0.4444)^3)

;*****************************************************************************
;  Get the radiometric response characterization uncertainty (RRCU),
;  and the radiometric response non-linearity (RRNL).
;*****************************************************************************

                        IF ifit LT 3 THEN BEGIN
                            mean_dL_fit[b,ih,id,is,ifit,rsr]=mean(dL_fit[dyr,b,ih,id,is,ifit,rsr])
                            stddev_dL_fit[b,ih,id,is,ifit,rsr]=stddev(dL_fit[dyr,b,ih,id,is,ifit,rsr])
                            
                            rrcu[b,ih,id,is,ifit,rsr,0]=mean_dL_fit[b,ih,id,is,ifit,rsr]+stddev_dL_fit[b,ih,id,is,ifit,rsr]
                            rrcu[b,ih,id,is,ifit,rsr,1]=sqrt(mean_dL_fit[b,ih,id,is,ifit,rsr]^2+stddev_dL_fit[b,ih,id,is,ifit,rsr]^2)
                        ENDIF
                        IF ifit GE 3 THEN BEGIN
                            abs_max_dL_fit[b,ih,id,is,ifit,rsr]=max(abs(dL_fit[dyr,b,ih,id,is,ifit,rsr]))
                            rrnl[b,ih,id,is,ifit,rsr]=abs_max_dL_fit[b,ih,id,is,ifit,rsr]/(L_max_TEB[b,0]*rg_1[b,ih,id,is,1,ifit,rsr])
                        ENDIF
                    ENDFOR
                ENDFOR
            ENDFOR
            IF ifit GT 2 THEN print,'GAIN ',ifit,rsr,STRING(9B),band_name_TEB[b],STRING(9B), $
              mean(rg_1[b,0,mindet[b]:detectors_TEB[b]-1,0:subsets_TEB[b]-1,1,ifit,rsr]), $
              mean(rg_1[b,1,mindet[b]:detectors_TEB[b]-1,0:subsets_TEB[b]-1,1,ifit,rsr])
            IF ifit GT 0 AND ifit LT 3 THEN print,'RRCU ',ifit,rsr,STRING(9B),band_name_TEB[b],STRING(9B), $
              mean(rrcu[b,0,mindet[b]:detectors_TEB[b]-1,0:subsets_TEB[b]-1,ifit,rsr,0]), $
              mean(rrcu[b,1,mindet[b]:detectors_TEB[b]-1,0:subsets_TEB[b]-1,ifit,rsr,0])
            IF ifit EQ 3 THEN print,'RRNL ',ifit,rsr,STRING(9B),band_name_TEB[b],STRING(9B), $
              mean(rrnl[b,0,mindet[b]:detectors_TEB[b]-1,0:subsets_TEB[b]-1,ifit,rsr]), $
              mean(rrnl[b,1,mindet[b]:detectors_TEB[b]-1,0:subsets_TEB[b]-1,ifit,rsr])
        ENDFOR
    ENDFOR
ENDFOR

save,/compress,ncollects,rg_1,dL_fit,rrcu,rrnl,filename=outfile

;*****************************************************************************
;  Ouput gains, rrcu, and rrnl to txt files.
;*****************************************************************************

outfile2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.txt'
outfile3=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.avg.txt'
outfile4=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrcu.max.txt'
outfile5=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrcu.avg.txt'
outfile6=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrnl.max.txt'
outfile7=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrnl.avg.txt'

fmt_1='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d)'
fmt_2='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile2,/get_lun
printf,lun,'Radiametric gains (with RSR)'
printf,lun,'Band  Detector  HAM  SUBSET   Gain(linear)   Gain(quad)  Gain(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            FOR idet=0, detectors_TEB[b]-1 DO BEGIN
                printf,lun,format=fmt_1,band_name_TEB[b],idet+1,ih,is,rg_1[b,ih,idet,0,1,3,0],rg_1[b,ih,idet,0,1,4,0],rg_1[b,ih,idet,is,1,5,0]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile3,/get_lun
printf,lun,'Radiametric gains (detector averaged with RSR)'
printf,lun,'Band  HAM  SUBSET   Gain(linear)   Gain(quad)   Gain(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name_TEB[b],ih,is, $
              mean(rg_1[b,ih,0:detectors_TEB[b]-1,is,1,3,0]),mean(rg_1[b,ih,0:detectors_TEB[b]-1,is,1,4,0]),mean(rg_1[b,ih,0:detectors_TEB[b]-1,is,1,5,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile4,/get_lun
printf,lun,'RRCU (worst case detector with RSR)'
printf,lun,'Band  HAM  SUBSET   RRCU(linear)  RRCU(quad)  RRCU(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name_TEB[b],ih,is, $
              max(rrcu[b,ih,0:detectors_TEB[b]-1,is,0,0,0]),max(rrcu[b,ih,0:detectors_TEB[b]-1,is,1,0,0]),max(rrcu[b,ih,0:detectors_TEB[b]-1,is,2,0,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun
		
openw,lun,outfile5,/get_lun
printf,lun,'RRCU (detector averaged with RSR)'
printf,lun,'Band  HAM  SUBSET   RRCU(linear)  RRCU(quad)  RRCU(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name_TEB[b],ih,is, $
              mean(rrcu[b,ih,0:detectors_TEB[b]-1,is,0,0,0]),mean(rrcu[b,ih,0:detectors_TEB[b]-1,is,1,0,0]),mean(rrcu[b,ih,0:detectors_TEB[b]-1,is,2,0,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile6,/get_lun
printf,lun,'RRNL (worst case detector with RSR)'
printf,lun,'Band  HAM  SUBSET  RRNL(linear)  RRNL(quad)  RRNL(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name_TEB[b],ih,is, $
              max(rrnl[b,ih,0:detectors_TEB[b]-1,is,0,0]),max(rrnl[b,ih,0:detectors_TEB[b]-1,is,1,0]),max(rrnl[b,ih,0:detectors_TEB[b]-1,is,2,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile7,/get_lun
printf,lun,'RRNL (detector averaged with RSR)'
printf,lun,'Band  HAM  SUBSET  RRNL(linear)  RRNL(quad)  RRNL(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name_TEB[b],ih,is, $
              mean(rrnl[b,ih,0:detectors_TEB[b]-1,is,0,0]),mean(rrnl[b,ih,0:detectors_TEB[b]-1,is,1,0]),mean(rrnl[b,ih,0:detectors_TEB[b]-1,is,2,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

outfile8=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.no_rsr.txt'
outfile9=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.avg.no_rsr.txt'
outfile10=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrcu.max.no_rsr.txt'
outfile11=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrcu.avg.no_rsr.txt'
outfile12=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrnl.max.no_rsr.txt'
outfile13=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrnl.avg.no_rsr.txt'

openw,lun,outfile8,/get_lun
printf,lun,'Radiametric gains (no RSR)'
printf,lun,'Band  Detector  HAM  SUBSET   Gain(linear)   Gain(quad)  Gain(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            FOR idet=0, detectors_TEB[b]-1 DO BEGIN
                printf,lun,format=fmt_1,band_name_TEB[b],idet+1,ih,is,rg_1[b,ih,idet,0,1,3,1],rg_1[b,ih,idet,0,1,4,1],rg_1[b,ih,idet,is,1,5,1]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile9,/get_lun
printf,lun,'Radiametric gains (detector averaged no RSR)'
printf,lun,'Band  HAM  SUBSET   Gain(linear)   Gain(quad)  Gain(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name_TEB[b],ih,is, $
              mean(rg_1[b,ih,0:detectors_TEB[b]-1,is,1,3,0]),mean(rg_1[b,ih,0:detectors_TEB[b]-1,is,1,4,0]),mean(rg_1[b,ih,0:detectors_TEB[b]-1,is,1,5,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile10,/get_lun
printf,lun,'RRCU (worst case detector no RSR)'
printf,lun,'Band  HAM  SUBSET   RRCU(linear)  RRCU(quad)  RRCU(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name_TEB[b],ih,is, $
              max(rrcu[b,ih,0:detectors_TEB[b]-1,is,0,1,0]),max(rrcu[b,ih,0:detectors_TEB[b]-1,is,1,1,0]),max(rrcu[b,ih,0:detectors_TEB[b]-1,is,2,1,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun
		
openw,lun,outfile11,/get_lun
printf,lun,'RRCU (detector averaged no RSR)'
printf,lun,'Band  HAM  SUBSET   RRCU(linear)  RRCU(quad)  RRCU(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name_TEB[b],ih,is, $
              mean(rrcu[b,ih,0:detectors_TEB[b]-1,is,0,1,0]),mean(rrcu[b,ih,0:detectors_TEB[b]-1,is,1,1,0]),mean(rrcu[b,ih,0:detectors_TEB[b]-1,is,2,1,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile12,/get_lun
printf,lun,'RRNL (worst case detector no RSR)'
printf,lun,'Band  HAM  SUBSET  RRNL(linear)  RRNL(quad)  RRNL(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name_TEB[b],ih,is, $
              max(rrnl[b,ih,0:detectors_TEB[b]-1,is,0,1]),max(rrnl[b,ih,0:detectors_TEB[b]-1,is,1,1]),max(rrnl[b,ih,0:detectors_TEB[b]-1,is,2,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile13,/get_lun
printf,lun,'RRNL (detector averaged no RSR)'
printf,lun,'Band  HAM  SUBSET  RRNL(linear)  RRNL(quad)  RRNL(cubic)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name_TEB[b],ih,is, $
              mean(rrnl[b,ih,0:detectors_TEB[b]-1,is,0,1]),mean(rrnl[b,ih,0:detectors_TEB[b]-1,is,1,1]),mean(rrnl[b,ih,0:detectors_TEB[b]-1,is,2,1])
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Ouput residual, coefficients, and dn@Lmax to txt files.
;*****************************************************************************

outfile14=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.residual.txt'
outfile15=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.coeff.txt'

fmt_3='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'
fmt_4='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile14,/get_lun
printf,lun,'BAND   COLLECT   HAM   SUBSET   DETECTOR   dn   dL_BCS   quadratic(RESIDUAL(FRACTIONAL)   RESIDUAL)   cubic(RESIDUAL(FRACTIONAL)   RESIDUAL)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR col=0, ncollects-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            FOR is=0, subsets_TEB[b]-1 DO BEGIN
                FOR id=0, detectors_TEB[b]-1 DO BEGIN
                    IF rad_BCS[col,b,ih,id,is,0] LT L_min_TEB[b] THEN continue
                    IF rad_BCS[col,b,ih,id,is,0] GT L_max_TEB[b] THEN continue
                    
                    printf,lun,format=fmt_3,band_name_TEB[b],col,ih,is,id+1,dn[col,b,ih,id,is,1,0],rad_BCS[col,b,ih,id,is,0],dL_fit[col,b,ih,id,is,1,0], $
                      dL_fit[col,b,ih,id,is,1,0]*rad_BCS[col,b,ih,id,is,0],dL_fit[col,b,ih,id,is,2,0],dL_fit[col,b,ih,id,is,2,0]*rad_BCS[col,b,ih,id,is,0]
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile15,/get_lun
printf,lun,'BAND   HAM   SUBSET   DETECTOR   quadratic(a0   a1   a2)   cubic(a0   a1   a2)'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets_TEB[b]-1 DO BEGIN
            FOR id=0, detectors_TEB[b]-1 DO BEGIN
                printf,lun,format=fmt_4,band_name_TEB[b],ih,is,id+1,rg_1[b,ih,id,is,0,1,0],rg_1[b,ih,id,is,1,1,0],rg_1[b,ih,id,is,2,1,0], $
                  rg_1[b,ih,id,is,0,2,0],rg_1[b,ih,id,is,1,2,0],rg_1[b,ih,id,is,2,2,0]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

outfile16=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.Lmax.txt'

fmt_5='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, d)'

openw,lun,outfile16,/get_lun
printf,lun,'BAND   HAM   SUBSET   DETECTOR   dn@Lmax'
FOR b=0, nbands_TEB-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets_TEB[b]-1 DO BEGIN
            FOR id=0, detectors_TEB[b]-1 DO BEGIN
                printf,lun,format=fmt_5,band_name_TEB[b],ih,is,id+1,rg_1[b,ih,id,is,0,4,0]+rg_1[b,ih,id,is,1,4,0]*L_max_TEB[b,0]+rg_1[b,ih,id,is,2,4,0]*L_max_TEB[b,0]^2
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

outfile17=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temporary.txt'

openw,lun,outfile17,/get_lun
FOR b=0, nbands_TEB-1 DO BEGIN
    xx=dblarr(nbands_TEB,maxdet,maxsubsets)
    FOR is=0, subsets_TEB[b]-1 DO BEGIN
        temp1=0
        temp2=0
        FOR id=0, detectors_TEB[b]-1 DO BEGIN
            xx[b,id,is]=max(dL_fit[0:ncollects-1,b,0,id,is,1,0])

        ENDFOR
	;	if b eq 3 or b eq 4 then print,b,is,mean(xx[b,mindet[b]:detectors_TEB[b]-1,is]),max(xx[b,mindet[b]:detectors_TEB[b]-1,is]),xx[b,mindet[b]:detectors_TEB[b]-1,is]
    ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

END
