;*****************************************************************************
;				NICST TOOL
;
;  Quicklooks temperature trending tool. Read the temperature data extracted using the LRV,
;  reformats it, and put it in the proper units. Read the temperature data from the
;  GSE files and reformats. Calculate the slopes of the temperatures over scan
;  and collect. Then write to a sav file and makes plots.
;  
;  Input:
;    1) Infile with directory containing LRV extracted files, GSE
;    files, and ADF files.
;
;  Output:
;    1) Temp of BCS, SVS, TMC, OMM, LWASP, MWASP, and CFPA as a function of collects
;        (labeled T_xxx[collects]).
;    2) Temp of RTA, CAV, SH, OBC, and HAM as a function of collects and HAM side
;        (labeled T_xxx[HAM,collects]).
;    3) Temp of RTA, CAV, SH, OBC, and HAM as a function of collects, HAM side, and scans
;        (labeled T_xxx_2[scans,HAM,collects]).
;    4) Standard deviation of OBC thermistors as a function of collects and HAM side
;        (labeled std_obc_temp[6,collects]).
;    5) Coefficients of straight line fit of temp versus scans: OMM 
;        (labeled Slope_temp_4[i,collects,coeff]); and VISNIR ASP, MWASP, and LWASP
;        (labeled Slope_temp_3[i,collects,coeff]); and VISNIR FPA, SMWIR FPA, and LWIR FPA 
;        (labeled Slope_temp_2[i,collects,coeff]); and RTA, CAV, SH, HAM, and OBC 
;        (labeled Slope_temp_1[i,collects,coeff]).
;    6) Coefficients of straight line fit of temp versus collects: OMM, BCS, SVS and TMC 
;        (labeled Slope_4[i,coeff]); and VISNIR ASP, MWASP, and LWASP
;        (labeled Slope_3[i,collects,coeff]); and VISNIR FPA, SMWIR FPA, and LWIR FPA 
;        (labeled Slope_2[i,collects,coeff]); and RTA, CAV, SH, HAM, and OBC 
;        (labeled Slope_1[i,collects,coeff]).
;
;  v1.0 - 12/18/2008 - Jeff McIntire
;*****************************************************************************

PRO RC5_read_LRV, infile

;*****************************************************************************
;  Read in input file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.
maxscan=intarr(1)               ; # of scans
sample1=intarr(1)               ; 1st M band sample
nsamples=intarr(1)              ; # of M band samples

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,maxscan
readf,lun,sample1
readf,lun,nsamples
Free_Lun,lun

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

nham=2
nscans=maxscan[0]

; Offset of RTA temperature from CAV temperature.
DT_rta=4

; Create the outfile name
a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

outfile=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.'

;*****************************************************************************
;  Read in the LRV extracted temperatures.
;*****************************************************************************

; Get list of .txt files from specified path. 
spawn, 'ls -C1 '+inpath+'LRV/*.txt',list_files

IF N_ELEMENTS(list_files) EQ 0 THEN goto,skipout
ncollects=N_ELEMENTS(list_files)

;*****************************************************************************
;  Temperature definitions.
;*****************************************************************************

; HAM side per scan.
ham=fltarr(nscans,ncollects)

; Initial temperatures for RTA, CAV (smae as RTA), SH, HAM, and OBC.
T_rta_1=dblarr(2,nscans,nham,ncollects)
T_sh_1=dblarr(2,nscans,nham,ncollects)
T_ham_1=dblarr(2,nscans,nham,ncollects)
T_obc_1=dblarr(6,nscans,nham,ncollects) 

; Average temperatures for RTA, CAV, SH, HAM, and OBC over inputs.
T_rta_2=dblarr(nscans,nham,ncollects)
T_cav_2=dblarr(nscans,nham,ncollects)
T_sh_2=dblarr(nscans,nham,ncollects)
T_ham_2=dblarr(nscans,nham,ncollects)
T_obc_2=dblarr(nscans,nham,ncollects)

; Average temperatures for RTA, CAV, SH, HAM, and OBC over scans.
T_rta=dblarr(nham,ncollects)	
T_cav=dblarr(nham,ncollects)	
T_sh=dblarr(nham,ncollects)
T_ham=dblarr(nham,ncollects)
T_obc=dblarr(nham,ncollects)

; Temperatures for the three focal planes for both CFPA and ASP, both
; read in values and scan averaged.
T_cfpa_1=dblarr(3,nscans,ncollects)
T_cfpa=dblarr(3,ncollects)
T_asp_1=dblarr(3,nscans,ncollects)
T_asp=dblarr(3,ncollects)

; Temperatures for both OMM temperatures, both read in values and scan averaged.
T_omm_1=dblarr(2,nscans,ncollects)
T_omm_2=dblarr(nscans,ncollects)
T_omm=dblarr(ncollects)

; Standard deviations of the OBC thermistors.
std_obc_temp=dblarr(6,ncollects)

; Definitions used in drift calulation.
f=findgen(nscans)
slope_temp_1=dblarr(ncollects,nham,5,2)

; Time of each scan.
time_scan=dblarr(nscans,ncollects)

;*****************************************************************************
;  Loop through collects.
;*****************************************************************************

FOR j=0, ncollects-1 DO BEGIN
    nlines=File_Lines(list_files[j])
    IF nlines EQ 0 THEN continue
    
    awa=''
    header1=strarr(1)
    header2=strarr(1)
    time=strarr(nlines)		; Time.
    scan=intarr(nlines)		; Scan number.
    LRV=intarr(nlines)		; LRV number.
    ham_elc=intarr(nlines)	; HAM side.
    EngUnits=fltarr(nlines)     ; Number extracted.
    
    openr,lun,list_files[j],/Get_Lun	
    readf,lun,header1
    readf,lun,header2
    
    FOR ix=0L, nlines-3 DO BEGIN
        readf,lun,awa
        a1=strsplit(awa,String(9B),/Extract)
        time(ix)=a1(2)
        scan(ix)=a1(3)
        LRV(ix)=a1(4)
        IF LRV[ix] NE 23206 AND LRV[ix] NE 21003 THEN EngUnits(ix)=DOUBLE(a1(7))
        IF LRV[ix] EQ 23206 THEN ham_elc(ix)=a1(6)

        ; Determine the time per scan.
        b1=strsplit(a1[2],' ',/Extract)
        c1=strsplit(b1[3],':',/Extract)
        time_scan[scan[ix]-1,j]=c1[0]*60.+c1[1]+c1[2]/60.
    ENDFOR
    Free_Lun,lun

;*****************************************************************************
;  Convert Celcius to Kelvin.
;*****************************************************************************

    FOR ix=0L, nlines-3 DO BEGIN
        IF LRV[ix] EQ 29024 THEN EngUnits[ix]=EngUnits[ix]+273.15
        IF LRV[ix] EQ 21028 THEN EngUnits[ix]=EngUnits[ix]+273.15
        IF LRV[ix] EQ 21029 THEN EngUnits[ix]=EngUnits[ix]+273.15
        IF LRV[ix] EQ 21030 THEN EngUnits[ix]=EngUnits[ix]+273.15
        IF LRV[ix] EQ 29017 THEN EngUnits[ix]=EngUnits[ix]+273.15
        IF LRV[ix] EQ 29016 THEN EngUnits[ix]=EngUnits[ix]+273.15
        IF LRV[ix] EQ 29014 THEN EngUnits[ix]=EngUnits[ix]+273.15
        IF LRV[ix] EQ 29011 THEN EngUnits[ix]=EngUnits[ix]+273.15
        IF LRV[ix] EQ 12021 THEN EngUnits[ix]=EngUnits[ix]+273.15
        IF LRV[ix] EQ 12022 THEN EngUnits[ix]=EngUnits[ix]+273.15
;        IF LRV[ix] EQ 32021 THEN EngUnits[ix]=EngUnits[ix]+273.15
;        IF LRV[ix] EQ 32022 THEN EngUnits[ix]=EngUnits[ix]+273.15
        IF LRV[ix] EQ 29005 THEN EngUnits[ix]=EngUnits[ix]+273.15
    ENDFOR

;*****************************************************************************
;  Get the HAM side per scan
;*****************************************************************************

    FOR ix=0L, nlines-3 DO BEGIN
        IF LRV[ix] EQ 23206 THEN ham[scan[ix]-1,j]=ham_elc[ix]
    ENDFOR

;*****************************************************************************
;  Write the temperatures to useful arrays.
;*****************************************************************************

    FOR ix=0L, nlines-3 DO BEGIN
        IF LRV[ix] EQ 29011 THEN T_rta_1[0,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 29014 THEN T_rta_1[1,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 29016 THEN T_sh_1[0,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 29017 THEN T_sh_1[1,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 12021 THEN T_ham_1[0,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 12022 THEN T_ham_1[1,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
;        IF LRV[ix] EQ 32021 THEN T_1[0,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
;        IF LRV[ix] EQ 32022 THEN T_1[1,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 2001 THEN T_obc_1[0,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 22002 THEN T_obc_1[1,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 2003 THEN T_obc_1[2,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 2004 THEN T_obc_1[3,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 22005 THEN T_obc_1[4,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 2006 THEN T_obc_1[5,scan[ix]-1,ham[scan[ix]-1,j],j]=EngUnits[ix]
        IF LRV[ix] EQ 27007 THEN T_cfpa_1[2,scan[ix]-1,j]=EngUnits[ix]
        IF LRV[ix] EQ 27009 THEN T_cfpa_1[1,scan[ix]-1,j]=EngUnits[ix]
        IF LRV[ix] EQ 27011 THEN T_cfpa_1[0,scan[ix]-1,j]=EngUnits[ix]
        IF LRV[ix] EQ 21028 THEN T_asp_1[2,scan[ix]-1,j]=EngUnits[ix]
        IF LRV[ix] EQ 21029 THEN T_asp_1[1,scan[ix]-1,j]=EngUnits[ix]
        IF LRV[ix] EQ 21030 THEN T_asp_1[0,scan[ix]-1,j]=EngUnits[ix]
        IF LRV[ix] EQ 29024 THEN T_omm_1[0,scan[ix]-1,j]=EngUnits[ix]
        IF LRV[ix] EQ 29005 THEN T_omm_1[1,scan[ix]-1,j]=EngUnits[ix]
    ENDFOR

;*****************************************************************************
;  Average the temperatures over blocks and scans.
;*****************************************************************************

    ; The CAV and RTA temperatures use the same inputs.
    ; Average over blocks, add 4 K to RTA temperature.
    FOR iham=0, nham-1 DO BEGIN
        FOR iscan=0, nscans-1 DO BEGIN
            IF T_rta_1[0,iscan,iham,j] NE 0 THEN T_rta_2[iscan,iham,j]=(T_rta_1[0,iscan,iham,j]+T_rta_1[1,iscan,iham,j])/2+DT_rta
        ENDFOR
    ENDFOR
    T_cav_2[*,*,j]=(T_rta_1[0,*,*,j]+T_rta_1[1,*,*,j])/2
    T_sh_2[*,*,j]=(T_sh_1[0,*,*,j]+T_sh_1[1,*,*,j])/2
    T_ham_2[*,*,j]=(T_ham_1[0,*,*,j]+T_ham_1[1,*,*,j])/2
    T_obc_2[*,*,j]=(T_obc_1[0,*,*,j]+T_obc_1[1,*,*,j]+T_obc_1[2,*,*,j]+T_obc_1[3,*,*,j]+T_obc_1[4,*,*,j]+T_obc_1[5,*,*,j])/6
    T_omm_2[*,j]=(T_omm_1[0,*,j]+T_omm_1[1,*,j])/2

    ; Average over scans.
    FOR iham=0, nham-1 DO BEGIN
        T_rta[iham,j]=sig_mean_no_kick(T_rta_2[*,iham,j],1)
        T_cav[iham,j]=sig_mean_no_kick(T_cav_2[*,iham,j],1)
        T_sh[iham,j]=sig_mean_no_kick(T_sh_2[*,iham,j],1)
        T_ham[iham,j]=sig_mean_no_kick(T_ham_2[*,iham,j],1)
        T_obc[iham,j]=sig_mean_no_kick(T_obc_2[*,iham,j],1)
    ENDFOR

    FOR k=0, 2 DO BEGIN
        T_cfpa[k,j]=sig_mean_no_kick(T_cfpa_1[k,*,j],1)
        T_asp[k,j]=sig_mean_no_kick(T_asp_1[k,*,j],1)
    ENDFOR

    T_omm[j]=sig_mean_no_kick(T_omm_2[*,j],1)
ENDFOR

; Make the first time zero.
time_scan[*,*]=time_scan[*,*]-time_scan[0,0]

FOR j=0, ncollects-1 DO BEGIN
    IF j NE 0 THEN BEGIN
        FOR iscan=0, nscans-1 DO BEGIN
            IF time_scan[iscan,j] LT time_scan[iscan,j-1] THEN time_scan[iscan,j]=time_scan[iscan,j]+24.*60.
        ENDFOR
    ENDIF
ENDFOR

skipout:

;*****************************************************************************
;  Read in the GSE extracted temperatures.
;*****************************************************************************

; Definitions.
T_bcs_1=dblarr(nscans,ncollects)
T_svs_1=dblarr(nscans,ncollects)
T_bcs_1_time=dblarr(nscans,ncollects)
T_svs_1_time=dblarr(nscans,ncollects)
T_bcs=dblarr(ncollects)
T_svs=dblarr(ncollects)
T_bcs_time=dblarr(ncollects)
T_svs_time=dblarr(ncollects)

; Get list of .txt files from specified path. 
spawn, 'ls -C1 '+inpath+'GSE/*.gse',list_files

IF N_ELEMENTS(list_files) EQ 0 THEN goto,skipout_1

;*****************************************************************************
;  Loop through collects.
;*****************************************************************************

FOR j=0, ncollects-1 DO BEGIN
    nlines=File_Lines(list_files[j])
    IF nlines EQ 0 THEN continue
    
    awa=''
    name=strarr(nlines)		; Name of quantity.
    time=strarr(nlines)		; Time.
    temp=dblarr(nlines)		; Temperature.
    
    openr,lun,list_files[j],/Get_Lun
    
    FOR ix=0L, nlines-1 DO BEGIN
        readf,lun,awa
        a1=strsplit(awa,String(9B),/Extract)
        name(ix)=a1(1)
        time(ix)=a1(2)
        temp(ix)=DOUBLE(a1(5))
    ENDFOR
    Free_Lun,lun

;*****************************************************************************
;  Convert Celcius to Kelvin.
;*****************************************************************************

    FOR ix=0L, nlines-1 DO BEGIN
        IF name[ix] EQ 'BS_BCS_EFT_TMP' THEN temp[ix]=temp[ix]+273.15
        IF name[ix] EQ 'BS_SVS_EFT_TMP' THEN temp[ix]=temp[ix]+273.15
    ENDFOR

;*****************************************************************************
;  Get time indices.
;*****************************************************************************

    k1=0
    k2=0
    FOR ix=0L, nlines-1 DO BEGIN
        IF name[ix] EQ 'BS_BCS_EFT_TMP' THEN BEGIN
            b1=strsplit(time[ix],':',/Extract)
            T_bcs_1_time[k1,j]=b1[1]*60.+b1[2]+b1[3]/60.
            k1=k1+1
        ENDIF
        IF name[ix] EQ 'BS_SVS_EFT_TMP' THEN BEGIN
            b1=strsplit(time[ix],':',/Extract)
            T_svs_1_time[k2,j]=b1[1]*60.+b1[2]+b1[3]/60.
            k2=k2+1
        ENDIF
    ENDFOR

;*****************************************************************************
;  Write the temperatures to useful arrays.
;*****************************************************************************

    k1=0
    k2=0
    FOR ix=0L, nlines-1 DO BEGIN
        IF name[ix] EQ 'BS_BCS_EFT_TMP' THEN BEGIN
            T_bcs_1[k1,j]=temp[ix]
            k1=k1+1
        ENDIF
        IF name[ix] EQ 'BS_SVS_EFT_TMP' THEN BEGIN
            T_svs_1[k2,j]=temp[ix]
            k2=k2+1
        ENDIF
    ENDFOR

;*****************************************************************************
;  Average over available data and detemine drifting over available scans.
;*****************************************************************************

    T_bcs[j]=sig_mean_no_kick(T_bcs_1[*,j],1)
    T_svs[j]=sig_mean_no_kick(T_svs_1[*,j],1)
    T_bcs_time[j]=sig_mean_no_kick(T_bcs_1_time[*,j],1)-sig_mean_no_kick(T_bcs_1_time[*,0],1)
    T_svs_time[j]=sig_mean_no_kick(T_svs_1_time[*,j],1)-sig_mean_no_kick(T_svs_1_time[*,0],1)

    IF j NE 0 THEN BEGIN
        IF T_bcs_time[j] LT T_bcs_time[j-1] THEN T_bcs_time[j]=T_bcs_time[j]+24.*60.
        IF T_svs_time[j] LT T_svs_time[j-1] THEN T_svs_time[j]=T_svs_time[j]+24.*60.
    ENDIF
ENDFOR

skipout_1:

;*****************************************************************************
;  Read in the TMC temperatures from the adf files.
;*****************************************************************************

; Definitions.
T_tmc=dblarr(ncollects)

; Get list of .txt files from specified path. 
spawn, 'ls -C1 '+inpath+'ADF/*.adf',list_files

IF N_ELEMENTS(list_files) LE 1 THEN goto,skipout_2

;*****************************************************************************
;  Loop through collects.
;*****************************************************************************

FOR j=0, ncollects-1 DO BEGIN
    nlines=File_Lines(list_files[j])
    IF nlines EQ 0 THEN continue
    
    awa=''
    header1=strarr(1)
    header2=strarr(1)
    name=strarr(1)
    
    openr,lun,list_files[j],/Get_Lun
    readf,lun,header1
    readf,lun,header2
    readf,lun,name
    
    T_tmc[j]=strmid(name[0],strpos(name[0],'=',/Reverse_Search)+2,5)+273.15
    
    Free_Lun,lun
ENDFOR

skipout_2:

;*****************************************************************************
;  Rewrite arrays to get rid of ham side dependence for plotting.
;*****************************************************************************

T=dblarr(nham,ncollects,5)
T_3=dblarr(nscans,ncollects,5)
T_4=dblarr(ncollects,5)
T_5=dblarr(6,nscans,ncollects)
FOR j=0, ncollects-1 DO BEGIN
    FOR iham=0, nham-1 DO BEGIN
        IF T_rta[iham,j] NE 0 THEN T_4[j,0]=T_rta[iham,j]
        IF T_cav[iham,j] NE 0 THEN T_4[j,1]=T_cav[iham,j]
        IF T_sh[iham,j] NE 0 THEN T_4[j,2]=T_sh[iham,j]
        IF T_ham[iham,j] NE 0 THEN T_4[j,3]=T_ham[iham,j]
        IF T_obc[iham,j] NE 0 THEN T_4[j,4]=T_obc[iham,j]
    ENDFOR
    FOR iscan=0, nscans-1 DO BEGIN
        FOR iham=0, nham-1 DO BEGIN
            IF T_rta_2[iscan,iham,j] NE 0 THEN T_3[iscan,j,0]=T_rta_2[iscan,iham,j]
            IF T_cav_2[iscan,iham,j] NE 0 THEN T_3[iscan,j,1]=T_cav_2[iscan,iham,j]
            IF T_sh_2[iscan,iham,j] NE 0 THEN T_3[iscan,j,2]=T_sh_2[iscan,iham,j]
            IF T_ham_2[iscan,iham,j] NE 0 THEN T_3[iscan,j,3]=T_ham_2[iscan,iham,j]
            IF T_obc_2[iscan,iham,j] NE 0 THEN T_3[iscan,j,4]=T_obc_2[iscan,iham,j]
            FOR m=0, 5 DO BEGIN
                IF T_obc_1[m,iscan,iham,j] NE 0 THEN T_5[m,iscan,j]=T_obc_1[m,iscan,iham,j]
            ENDFOR
        ENDFOR
    ENDFOR

    ; Take the standard deviations of OBC thermistors.
    FOR m=0, 5 DO BEGIN
        std_obc_temp[m,j]=sig_mean(T_5[m,*,j],3,2)
    ENDFOR
ENDFOR

; Make arrays more compact to lessen plotting routine size.
FOR j=0, ncollects-1 DO BEGIN
    FOR iham=0, nham-1 DO BEGIN
        T[iham,j,0]=T_rta[iham,j]
        T[iham,j,1]=T_cav[iham,j]
        T[iham,j,2]=T_sh[iham,j]
        T[iham,j,3]=T_ham[iham,j]
        T[iham,j,4]=T_obc[iham,j]
    ENDFOR
ENDFOR

; Determine the slopes.
f=findgen(nscans)
slope_temp_1=dblarr(ncollects,5,2)
slope_temp_2=dblarr(ncollects,3,2)
slope_temp_3=dblarr(ncollects,3,2)
slope_temp_4=dblarr(ncollects,1,2)
slope_1=dblarr(5,2)
slope_2=dblarr(3,2)
slope_3=dblarr(3,2)
slope_4=dblarr(4,2)
FOR j=0, ncollects-1 DO BEGIN

    ; Determine the slopes over scan.
    FOR k=0, 4 DO BEGIN
        slope_temp_1[j,k,*]=poly_fit(f[0:nscans-1],T_3[0:nscans-1,j,k],1)
    ENDFOR

    FOR k=0, 2 DO BEGIN
        slope_temp_2[j,k,*]=poly_fit(f[0:nscans-1],T_cfpa_1[k,0:nscans-1,j],1)
    ENDFOR

    FOR k=0, 2 DO BEGIN
        slope_temp_3[j,k,*]=poly_fit(f[0:nscans-1],T_asp_1[k,0:nscans-1,j],1)
    ENDFOR

    slope_temp_4[j,0,*]=poly_fit(f[0:nscans-1],T_omm_2[0:nscans-1,j],1)

    ; Determine the slopes over collect.
    FOR k=0, 4 DO BEGIN
        slope_1[k,*]=poly_fit(f[0:ncollects-1],T_4[0:ncollects-1,k],1)
    ENDFOR

    FOR k=0, 2 DO BEGIN
    slope_2[k,*]=poly_fit(f[0:ncollects-1],T_cfpa[k,0:ncollects-1],1)
    ENDFOR

    FOR k=0, 2 DO BEGIN
    slope_3[k,*]=poly_fit(f[0:ncollects-1],T_asp[k,0:ncollects-1],1)
    ENDFOR

    slope_4[0,*]=poly_fit(f[0:ncollects-1],T_omm[0:ncollects-1],1)
    slope_4[1,*]=poly_fit(f[0:ncollects-1],T_bcs[0:ncollects-1],1)
    slope_4[2,*]=poly_fit(f[0:ncollects-1],T_svs[0:ncollects-1],1)
    slope_4[3,*]=poly_fit(f[0:ncollects-1],T_tmc[0:ncollects-1],1)
ENDFOR

;*****************************************************************************
;  Write to a save file.
;*****************************************************************************

save,/compress,ncollects,ham,T_bcs,T_svs,T_tmc,T_asp,T_asp_1,T_omm,T_omm_2,T_cfpa,T_cfpa_1,T_rta,T_rta_2,T_cav, $
  T_cav_2,T_sh,T_sh_2,T_ham,T_ham_2,T_obc,T_obc_2,std_obc_temp,slope_temp_1,slope_temp_2,slope_temp_3, $
  slope_temp_4,slope_1,slope_2,slope_3,slope_4,filename=outfile+'temp.sav'

;*****************************************************************************
;  Write slopes to a text file.
;*****************************************************************************

fmt='(a15, 5x, i2, 5x, d)'
fmt2='(a15, 5x, d)'

openw,lun,outfile+'temp.slope.txt',/get_lun
printf,lun,'Slopes of temperatures over scan (variation of more than 0.05 degrees over 100 scans)'
printf,lun,'Temperature    Collect   Slope'
FOR j=0, ncollects-1 DO BEGIN
    IF slope_temp_1[j,0,1] GT 0.0005 THEN printf,lun,format=fmt,'T_rta ',j,slope_temp_1[j,0,1]
    IF slope_temp_1[j,1,1] GT 0.0005 THEN printf,lun,format=fmt,'T_cav ',j,slope_temp_1[j,1,1]
    IF slope_temp_1[j,2,1] GT 0.0005 THEN printf,lun,format=fmt,'T_sh ',j,slope_temp_1[j,2,1]
    IF slope_temp_1[j,3,1] GT 0.0005 THEN printf,lun,format=fmt,'T_ham ',j,slope_temp_1[j,3,1]
    IF slope_temp_1[j,4,1] GT 0.0005 THEN printf,lun,format=fmt,'T_obc ',j,slope_temp_1[j,4,1]
    IF slope_temp_2[j,0,1] GT 0.0005 THEN printf,lun,format=fmt,'VisNIR T_fpa ',j,slope_temp_1[j,0,1]
    IF slope_temp_2[j,1,1] GT 0.0005 THEN printf,lun,format=fmt,'SMWIR T_cfpa ',j,slope_temp_1[j,1,1]
    IF slope_temp_2[j,2,1] GT 0.0005 THEN printf,lun,format=fmt,'LWIR T_cfpa ',j,slope_temp_1[j,2,1]
    IF slope_temp_3[j,0,1] GT 0.0005 THEN printf,lun,format=fmt,'VisNIR T_asp ',j,slope_temp_1[j,0,1]
    IF slope_temp_3[j,1,1] GT 0.0005 THEN printf,lun,format=fmt,'SMWIR T_asp ',j,slope_temp_1[j,1,1]
    IF slope_temp_3[j,2,1] GT 0.0005 THEN printf,lun,format=fmt,'LWIR T_asp ',j,slope_temp_1[j,2,1]
    IF slope_temp_4[j,0,1] GT 0.0005 THEN printf,lun,format=fmt,'T_omm ',j,slope_temp_1[j,0,1]
ENDFOR
printf,lun,'Slopes of temperatures over collect (variation of more than 1 degree over '+strtrim(string(ncollects),2)+' collects)'
printf,lun,'Temperature    Slope'
IF slope_1[0,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'T_rta ',slope_1[0,1]
IF slope_1[1,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'T_cav ',slope_1[1,1]
IF slope_1[2,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'T_sh ',slope_1[2,1]
IF slope_1[3,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'T_ham ',slope_1[3,1]
IF slope_1[4,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'T_obc ',slope_1[4,1]
IF slope_2[0,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'VisNIR T_fpa ',slope_2[0,1]
IF slope_2[1,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'SMWIR T_cfpa ',slope_2[1,1]
IF slope_2[2,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'LWIR T_cfpa ',slope_2[2,1]
IF slope_3[0,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'VisNIR T_asp ',slope_3[0,1]
IF slope_3[1,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'SMWIR T_asp ',slope_3[1,1]
IF slope_3[2,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'LWIR T_asp ',slope_3[2,1]
IF slope_4[0,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'T_omm ',slope_4[0,1]
IF slope_4[1,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'T_bcs ',slope_4[1,1]
IF slope_4[2,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'T_svs ',slope_4[2,1]
IF slope_4[3,1] GT 1/double(ncollects) THEN printf,lun,format=fmt2,'T_tmc ',slope_4[3,1]
free_lun,lun

;*****************************************************************************
;  Set plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outfile+'temp.ps'
device,/landscape,filename=outfname,/color

D=findgen(4000)

temperatures1=['T_rta','T_cav','T_sh','T_ham','T_obc']
temperatures2=['VisNIR T_fpa','SMWIR T_cfpa','LWIR T_cfpa']
temperatures3=['VisNIR T_asp','SMWIR T_asp','LWIR T_asp']

;*****************************************************************************
;  BCS, SVS, and TMC temperatures.
;*****************************************************************************

!p.multi=[0,1,1]
!p.position=0

title='T_bcs vs Collect'
plot,D[0:ncollects+1],xtit='Collect',ytit='T_bcs (K)',yrange=[min(T_bcs[*])-0.5,max(T_bcs[*])+0.5],charsize=1.2,charthick=5,thick=3,/nodata

oplot,D[0:ncollects-1]+1,T_bcs[0:ncollects-1],symsize=1,color=lcol[0],psym=symbol[0],thick=3
oplot,D[0:ncollects-1]+1,T_bcs[0:ncollects-1],symsize=1,color=lcol[0],thick=3
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

title='T_svs vs Collect'
plot,D[0:ncollects+1],xtit='Collect',ytit='T_svs (K)',yrange=[min(T_svs[*])-0.5,max(T_svs[*])+0.5],charsize=1.2,charthick=5,thick=3,/nodata

oplot,D[0:ncollects-1]+1,T_svs[0:ncollects-1],symsize=1,color=lcol[0],psym=symbol[0],thick=3
oplot,D[0:ncollects-1]+1,T_svs[0:ncollects-1],symsize=1,color=lcol[0],thick=3
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

title='T_bcs vs Time'
plot,T_bcs_time[0:ncollects-1],xtit='Time (min)',ytit='T_bcs (K)',yrange=[min(T_bcs[*])-0.5,max(T_bcs[*])+0.5], $
  xrange=[min(T_bcs_time[*])-1,max(T_bcs_time[*])+1],charsize=1.2,charthick=5,thick=3,/nodata

oplot,T_bcs_time[0:ncollects-1],T_bcs[0:ncollects-1],symsize=1,color=lcol[0],psym=symbol[0],thick=3
oplot,T_bcs_time[0:ncollects-1],T_bcs[0:ncollects-1],symsize=1,color=lcol[0],thick=3
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

title='T_svs vs Time'
plot,T_svs_time[0:ncollects-1],xtit='Time (min)',ytit='T_svs (K)',yrange=[min(T_svs[*])-0.5,max(T_svs[*])+0.5], $
  xrange=[min(T_svs_time[*])-1,max(T_svs_time[*])+1],charsize=1.2,charthick=5,thick=3,/nodata

oplot,T_svs_time[0:ncollects-1],T_svs[0:ncollects-1],symsize=1,color=lcol[0],psym=symbol[0],thick=3
oplot,T_svs_time[0:ncollects-1],T_svs[0:ncollects-1],symsize=1,color=lcol[0],thick=3
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

IF mean(T_tmc[*]) EQ 0 THEN goto,skip_tmc

title='T_tmc vs Collect'
plot,D[0:ncollects+1],xtit='Collect',ytit='T_tmc (K)',yrange=[min(T_tmc[*])-0.5,max(T_tmc[*])+0.5],charsize=1.2,charthick=5,thick=3,/nodata

oplot,D[0:ncollects-1]+1,T_tmc[0:ncollects-1],symsize=1,color=lcol[0],psym=symbol[0],thick=3
oplot,D[0:ncollects-1]+1,T_tmc[0:ncollects-1],symsize=1,color=lcol[0],thick=3
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

skip_tmc:

;*****************************************************************************
;  RTA, CAV, SH, HAM, and OBC temperatures.
;*****************************************************************************

!p.multi=[0,1,1]
!p.position=0

FOR k=0, 4 DO BEGIN
    title=temperatures1[k]+' vs Scan and Collect'
    plot,D[0:nscans*ncollects],xtit='Collect*scan',ytit=temperatures1[k]+' (K)',yrange=[min(T[0,*,k])-0.5,max(T[0,*,k])+0.5], $
      charsize=1.2,charthick=5,thick=3,/nodata
    
    FOR col=0, ncollects-1 DO BEGIN
        oplot,D[nscans*col:nscans*(col+1)-1],T_3[0:nscans-1,col,k],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    ENDFOR
    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
ENDFOR

FOR k=0, 4 DO BEGIN
    title=temperatures1[k]+' vs Time'
    plot,time_scan[0:nscans-1,0],xtit='Time (min)',ytit=temperatures1[k]+' (K)',yrange=[min(T[0,*,k])-0.5,max(T[0,*,k])+0.5], $
      xrange=[min(time_scan[*,0])-1,max(time_scan[*,ncollects-1])+1],charsize=1.2,charthick=5,thick=3,/nodata
    
    FOR col=0, ncollects-1 DO BEGIN
        oplot,time_scan[0:nscans-1,col],T_3[0:nscans-1,col,k],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    ENDFOR
    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
ENDFOR

FOR k=0, 4 DO BEGIN
    !p.multi=[0,3,3]
    !p.position=0
    
    title=temperatures1[k]+' vs Scan'
    FOR col=0, ncollects-1 DO BEGIN
        title1='Collect='+strtrim(string(col+1),2)
        
        ymax=max(T_3[*,col,k])
        plot,T_3[*,col,k],xtit='Scan',ytit=temperatures1[k]+' (K)',yrange=[min(T_3[*,col,k])-0.05,ymax+0.05],title=title1,charsize=1.2,charthick=5,thick=3,/nodata
        
        oplot,T_3[*,col,k],symsize=1,color=lcol[col],psym=symbol[col],thick=3
        XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
    ENDFOR
ENDFOR

;*****************************************************************************
;  OBC thermistor standard deviations
;*****************************************************************************

!p.multi=[0,1,1]
!p.position=0

thermistor=['1','2','3','4','5','6']
title='OBC thermistor standard deviations vs Collect'
plot,D[0:ncollects],xtit='Collect',ytit='Standard deviation',xrange=[0,ncollects+1],yrange=[0.0,max(std_obc_temp[*,*])],charsize=1.2,charthick=5,thick=3,/nodata
    
FOR k=0, 5 DO BEGIN
    oplot,D[*]+1,std_obc_temp[k,*],symsize=1,color=lcol[k],psym=symbol[k],thick=3
    oplot,D[*]+1,std_obc_temp[k,*],symsize=1,color=lcol[k],thick=3
ENDFOR
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
set_legend,1,thermistor,lcol,symbol

;*****************************************************************************
;  FPA temperatures.
;*****************************************************************************

FOR k=0, 2 DO BEGIN
    !p.multi=[0,1,1]
    !p.position=0
    
    title=temperatures2[k]+' vs Scan and Collect'
    plot,D[0:nscans*ncollects],xtit='Collect*scan',ytit=temperatures2[k]+' (K)',yrange=[min(T_cfpa[k,*])-.5,max(T_cfpa[k,*])+.5],charsize=1.2,charthick=5,thick=3,/nodata
    
    FOR col=0, ncollects-1 DO BEGIN
        oplot,D[nscans*col:nscans*(col+1)-1],T_cfpa_1[k,0:nscans-1,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    ENDFOR
    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

    title=temperatures2[k]+' vs Time'
    plot,time_scan[0:nscans-1,0],xtit='Time (min)',ytit=temperatures2[k]+' (K)',yrange=[min(T_cfpa[k,*])-.5,max(T_cfpa[k,*])+.5], $
      xrange=[min(time_scan[*,0])-1,max(time_scan[*,ncollects-1])+1],charsize=1.2,charthick=5,thick=3,/nodata
    
    FOR col=0, ncollects-1 DO BEGIN
        oplot,time_scan[0:nscans-1,col],T_cfpa_1[k,0:nscans-1,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    ENDFOR
    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
    
    !p.multi=[0,3,3]
    !p.position=0
    
    title=temperatures2[k]+' vs Scan'
    FOR col=0, ncollects-1 DO BEGIN
        title1='Collect='+strtrim(string(col+1),2)
        
        ymax=max(T_cfpa_1[k,*,col])
        plot,T_cfpa_1[k,*,col],xtit='Scan',ytit=temperatures2[k]+' (K)',yrange=[min(T_cfpa_1[k,*,col])-0.05,ymax+0.05],title=title1,charsize=1.2,charthick=5,thick=3,/nodata
        
        oplot,T_cfpa_1[k,*,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    ENDFOR
    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
ENDFOR

FOR col=0, ncollects-1 DO BEGIN
    !p.multi=[0,2,2]
    !p.position=0
    
    title='T_cpfa vs Scan: Collect='+strtrim(string(col+1),2)
    title1='VisNIR'
    ymax=max(T_cfpa_1[0,*,col])
    plot,T_cfpa_1[0,*,col],xtit='Scan',ytit='VisNIR T_fpa (K)',yrange=[min(T_cfpa_1[0,*,col])-0.05,ymax+0.05],title=title1,charsize=1.2,charthick=5,thick=3,/nodata
    
    oplot,T_cfpa_1[0,*,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3

    title1='SMWIR'
    ymax=max(T_cfpa_1[1,*,col])
    plot,T_cfpa_1[1,*,col],xtit='Scan',ytit='SMWIR T_cfpa (K)',yrange=[min(T_cfpa_1[1,*,col])-0.05,ymax+0.05],title=title1,charsize=1.2,charthick=5,thick=3,/nodata
    
    oplot,T_cfpa_1[1,*,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    
    title1='LWIR'
    ymax=max(T_cfpa_1[2,*,col])
    plot,T_cfpa_1[2,*,col],xtit='Scan',ytit='LWIR T_cfpa (K)',yrange=[min(T_cfpa_1[2,*,col])-0.05,ymax+0.05],title=title1,charsize=1.2,charthick=5,thick=3,/nodata

    oplot,T_cfpa_1[2,*,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
ENDFOR

;*****************************************************************************
;  ASP temperatures.
;*****************************************************************************

FOR k=0, 2 DO BEGIN
    !p.multi=[0,1,1]
    !p.position=0
    
    title=temperatures3[k]+' vs Scan and Collect'
    plot,D[0:nscans*ncollects],xtit='Collect*scan',ytit=temperatures3[k]+' (K)',yrange=[min(T_asp[k,*])-.5,max(T_asp[k,*])+.5],charsize=1.2,charthick=5,thick=3,/nodata
    
    FOR col=0, ncollects-1 DO BEGIN
        oplot,D[nscans*col:nscans*(col+1)-1],T_asp_1[k,0:nscans-1,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    ENDFOR
    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
    
    title=temperatures3[k]+' vs Time'
    plot,time_scan[0:nscans-1,0],xtit='Time (min)',ytit=temperatures3[k]+' (K)',yrange=[min(T_asp[k,*])-.5,max(T_asp[k,*])+.5], $
      xrange=[min(time_scan[*,0])-1,max(time_scan[*,ncollects-1])+1],charsize=1.2,charthick=5,thick=3,/nodata
    
    FOR col=0, ncollects-1 DO BEGIN
        oplot,time_scan[0:nscans-1,col],T_asp_1[k,0:nscans-1,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    ENDFOR
    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

    !p.multi=[0,3,3]
    !p.position=0
    
    title=temperatures3[k]+' vs Scan'
    FOR col=0, ncollects-1 DO BEGIN
        title1='Collect='+strtrim(string(col+1),2)
        
        ymax=max(T_asp_1[k,*,col])
        plot,T_asp_1[k,*,col],xtit='Scan',ytit=temperatures3[k]+' (K)',yrange=[min(T_asp_1[k,*,col])-0.05,ymax+0.05],title=title1,charsize=1.2,charthick=5,thick=3,/nodata
        
        oplot,T_asp_1[k,*,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    ENDFOR
    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
ENDFOR

FOR col=0, ncollects-1 DO BEGIN
    !p.multi=[0,2,2]
    !p.position=0
    
    title='T_asp vs Scan: Collect='+strtrim(string(col+1),2)
    title1='VisNIR'
    ymax=max(T_asp_1[0,*,col])
    plot,T_asp_1[0,*,col],xtit='Scan',ytit='VisNIR T_asp (K)',yrange=[min(T_asp_1[0,*,col])-0.05,ymax+0.05],title=title1,charsize=1.2,charthick=5,thick=3,/nodata
    
    oplot,T_asp_1[0,*,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3

    title1='SMWIR'
    ymax=max(T_asp_1[1,*,col])
    plot,T_asp_1[1,*,col],xtit='Scan',ytit='SMWIR T_asp (K)',yrange=[min(T_asp_1[1,*,col])-0.05,ymax+0.05],title=title1,charsize=1.2,charthick=5,thick=3,/nodata
    
    oplot,T_asp_1[1,*,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    
    title1='LWIR'
    ymax=max(T_asp_1[2,*,col])
    plot,T_asp_1[2,*,col],xtit='Scan',ytit='LWIR T_asp (K)',yrange=[min(T_asp_1[2,*,col])-0.05,ymax+0.05],title=title1,charsize=1.2,charthick=5,thick=3,/nodata

    oplot,T_asp_1[2,*,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
ENDFOR

;*****************************************************************************
;  OMM temperature.
;*****************************************************************************

!p.multi=[0,1,1]
!p.position=0

title='T_omm vs Scan and Collect'
plot,D[0:nscans*ncollects],xtit='Collect*scan',ytit='T_omm (K)',yrange=[min(T_omm[*])-.5,max(T_omm[*])+.5],charsize=1.2,charthick=5,thick=3,/nodata

FOR col=0, ncollects-1 DO BEGIN
    oplot,D[nscans*col:nscans*(col+1)-1],T_omm_2[0:nscans-1,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
ENDFOR
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

title='T_omm vs Time'
plot,time_scan[0:nscans-1,0],xtit='Time (min)',ytit='T_omm (K)',yrange=[min(T_omm[*])-.5,max(T_omm[*])+.5], $
      xrange=[min(time_scan[*,0])-1,max(time_scan[*,ncollects-1])+1],charsize=1.2,charthick=5,thick=3,/nodata

FOR col=0, ncollects-1 DO BEGIN
    oplot,time_scan[0:nscans-1,col],T_omm_2[0:nscans-1,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
ENDFOR
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

!p.multi=[0,3,3]
!p.position=0

title='T_omm vs Scan'
FOR col=0, ncollects-1 DO BEGIN
    title1='Collect='+strtrim(string(col+1),2)
    
    ymax=max(T_omm_2[*,col])
    plot,T_omm_2[*,col],xtit='Scan',ytit='T_omm (K)',yrange=[min(T_omm_2[*,col])-0.05,ymax+0.05],title=title1,charsize=1.2,charthick=5,thick=3,/nodata
    
    oplot,T_omm_2[*,col],symsize=1,color=lcol[col],psym=symbol[col],thick=3
    XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
ENDFOR

device,/close

END
